#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2013, Adam Miller <maxamillion@fedoraproject.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: firewalld
short_description: Manage arbitrary ports/services with firewalld
description:
  - This module allows for addition or deletion of services and ports (either TCP or UDP) in either running or permanent firewalld rules.
options:
  service:
    description:
      - Name of a service to add/remove to/from firewalld.
      - The service must be listed in output of C(firewall-cmd --get-services).
    type: str
  protocol:
    description:
      - Name of a protocol to add/remove to/from firewalld.
    type: str
  port:
    description:
      - Name of a port or port range to add/remove to/from firewalld.
      - Must be in the form PORT/PROTOCOL or PORT-PORT/PROTOCOL for port ranges.
    type: str
  port_forward:
    description:
      - Port and protocol to forward using firewalld.
    type: list
    elements: dict
    suboptions:
      port:
        type: str
        required: true
        description:
          - Source port to forward from.
      proto:
        type: str
        required: true
        description:
          - protocol to forward.
        choices: [udp, tcp]
      toport:
        type: str
        required: true
        description:
          - destination port.
      toaddr:
        type: str
        description:
          - Optional address to forward to.
  rich_rule:
    description:
      - Rich rule to add/remove to/from firewalld.
      - See L(Syntax for firewalld rich language rules,https://firewalld.org/documentation/man-pages/firewalld.richlanguage.html).
    type: str
  source:
    description:
      - The source/network you would like to add/remove to/from firewalld.
    type: str
  interface:
    description:
      - The interface you would like to add/remove to/from a zone in firewalld.
    type: str
  icmp_block:
    description:
      - The ICMP block you would like to add/remove to/from a zone in firewalld.
    type: str
  icmp_block_inversion:
    description:
      - Enable/Disable inversion of ICMP blocks for a zone in firewalld.
      - Note that the option type is changed to bool in ansible.posix version 2.0.0 and later.
    type: bool
  zone:
    description:
      - The firewalld zone to add/remove to/from.
      - Note that the default zone can be configured per system but V(public) is default from upstream.
      - Available choices can be extended based on per-system configs, listed here are "out of the box" defaults.
      - Possible values include V(block), V(dmz), V(drop), V(external), V(home), V(internal), V(public), V(trusted), V(work).
    type: str
  permanent:
    description:
      - Whether to apply this change to the permanent firewalld configuration.
      - As of Ansible 2.3, permanent operations can operate on firewalld configs when it is not running (requires firewalld >= 0.3.9).
      - Note that if this is V(false), O(immediate=true) by default.
    type: bool
    default: false
  immediate:
    description:
      - Whether to apply this change to the runtime firewalld configuration.
      - Defaults to V(true) if O(permanent=false).
    type: bool
    default: false
  state:
    description:
      - Enable or disable a setting.
      - 'For ports: Should this port accept (V(enabled)) or reject (V(disabled)) connections.'
      - The states V(present) and V(absent) can only be used in zone level operations (i.e. when no other parameters but zone and state are set).
    type: str
    required: true
    choices: [ absent, disabled, enabled, present ]
  timeout:
    description:
      - The amount of time in seconds the rule should be in effect for when non-permanent.
    type: int
    default: 0
  forward:
    description:
      - The forward setting you would like to enable/disable to/from zones within firewalld.
      - This option only is supported by firewalld v0.9.0 or later.
      - Note that the option type is changed to bool in ansible.posix version 2.0.0 and later.
    type: bool
  masquerade:
    description:
      - The masquerade setting you would like to enable/disable to/from zones within firewalld.
      - Note that the option type is changed to bool in ansible.posix version 2.0.0 and later.
    type: bool
  offline:
    description:
      - Ignores O(immediate) if O(permanent=true) and firewalld is not running.
    type: bool
    default: false
  target:
    description:
      - firewalld Zone target.
      - If O(state=absent), this will reset the target to default.
    choices: [ default, ACCEPT, DROP, "%%REJECT%%" ]
    type: str
    version_added: 1.2.0
notes:
  - Not tested on any Debian based system.
  - Requires the python2 bindings of firewalld, which may not be installed by default.
  - For distributions where the python2 firewalld bindings are unavailable (e.g Fedora 28 and later) you will have to set the
    ansible_python_interpreter for these hosts to the python3 interpreter path and install the python3 bindings.
  - Zone transactions (creating, deleting) can be performed by using only the zone and state parameters "present" or "absent".
    Note that zone transactions must explicitly be permanent. This is a limitation in firewalld.
    This also means that you will have to reload firewalld after adding a zone that you wish to perform immediate actions on.
    The module will not take care of this for you implicitly because that would undo any previously performed immediate actions which were not
    permanent. Therefore, if you require immediate access to a newly created zone it is recommended you reload firewalld immediately after the zone
    creation returns with a changed state and before you perform any other immediate, non-permanent actions on that zone.
  - This module needs C(python-firewall) or C(python3-firewall) on managed nodes.
    It is usually provided as a subset with C(firewalld) from the OS distributor for the OS default Python interpreter.
requirements:
- firewalld >= 0.9.0
- python-firewall >= 0.9.0
author:
- Adam Miller (@maxamillion)
'''

EXAMPLES = r'''
- name: Permanently enable https service, also enable it immediately if possible
  ansible.posix.firewalld:
    service: https
    state: enabled
    permanent: true
    immediate: true
    offline: true

- name: Permit traffic in default zone for https service
  ansible.posix.firewalld:
    service: https
    permanent: true
    state: enabled

- name: Permit ospf traffic
  ansible.posix.firewalld:
    protocol: ospf
    permanent: true
    state: enabled

- name: Do not permit traffic in default zone on port 8081/tcp
  ansible.posix.firewalld:
    port: 8081/tcp
    permanent: true
    state: disabled

- name: Permit traffic in default zone on port 161-162/ucp
  ansible.posix.firewalld:
    port: 161-162/udp
    permanent: true
    state: enabled

- name: Permit traffic in dmz zone on http service
  ansible.posix.firewalld:
    zone: dmz
    service: http
    permanent: true
    state: enabled

- name: Enable FTP service with rate limiting using firewalld rich rule
  ansible.posix.firewalld:
    rich_rule: rule service name="ftp" audit limit value="1/m" accept
    permanent: true
    state: enabled

- name: Allow traffic from 192.0.2.0/24 in internal zone
  ansible.posix.firewalld:
    source: 192.0.2.0/24
    zone: internal
    state: enabled

- name: Assign eth2 interface to trusted zone
  ansible.posix.firewalld:
    zone: trusted
    interface: eth2
    permanent: true
    state: enabled

- name: Enable forwarding in internal zone
  ansible.posix.firewalld:
    forward: true
    state: enabled
    permanent: true
    zone: internal

- name: Enable masquerade in dmz zone
  ansible.posix.firewalld:
    masquerade: true
    state: enabled
    permanent: true
    zone: dmz

- name: Create custom zone if not already present
  ansible.posix.firewalld:
    zone: custom
    state: present
    permanent: true

- name: Enable ICMP block inversion in drop zone
  ansible.posix.firewalld:
    zone: drop
    state: enabled
    permanent: true
    icmp_block_inversion: true

- name: Block ICMP echo requests in drop zone
  ansible.posix.firewalld:
    zone: drop
    state: enabled
    permanent: true
    icmp_block: echo-request

- name: Set internal zone target to ACCEPT
  ansible.posix.firewalld:
    zone: internal
    state: present
    permanent: true
    target: ACCEPT

- name: Redirect port 443 to 8443 with Rich Rule
  ansible.posix.firewalld:
    rich_rule: rule family=ipv4 forward-port port=443 protocol=tcp to-port=8443
    zone: public
    permanent: true
    immediate: true
    state: enabled
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.posix.plugins.module_utils.firewalld import FirewallTransaction, fw_offline

try:
    from firewall.client import Rich_Rule
    from firewall.client import FirewallClientZoneSettings
except ImportError:
    # The import errors are handled via FirewallTransaction, don't need to
    # duplicate that here
    pass


class IcmpBlockTransaction(FirewallTransaction):
    """
    IcmpBlockTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(IcmpBlockTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, icmp_block, timeout):
        return icmp_block in self.fw.getIcmpBlocks(self.zone)

    def get_enabled_permanent(self, icmp_block, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        return icmp_block in fw_settings.getIcmpBlocks()

    def set_enabled_immediate(self, icmp_block, timeout):
        self.fw.addIcmpBlock(self.zone, icmp_block, timeout)

    def set_enabled_permanent(self, icmp_block, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addIcmpBlock(icmp_block)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, icmp_block, timeout):
        self.fw.removeIcmpBlock(self.zone, icmp_block)

    def set_disabled_permanent(self, icmp_block, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeIcmpBlock(icmp_block)
        self.update_fw_settings(fw_zone, fw_settings)


class IcmpBlockInversionTransaction(FirewallTransaction):
    """
    IcmpBlockInversionTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(IcmpBlockInversionTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self):
        if self.fw.queryIcmpBlockInversion(self.zone) is True:
            return True
        else:
            return False

    def get_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        if fw_settings.getIcmpBlockInversion() is True:
            return True
        else:
            return False

    def set_enabled_immediate(self):
        self.fw.addIcmpBlockInversion(self.zone)

    def set_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setIcmpBlockInversion(True)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self):
        self.fw.removeIcmpBlockInversion(self.zone)

    def set_disabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setIcmpBlockInversion(False)
        self.update_fw_settings(fw_zone, fw_settings)


class ServiceTransaction(FirewallTransaction):
    """
    ServiceTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(ServiceTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, service, timeout):
        if service in self.fw.getServices(self.zone):
            return True
        else:
            return False

    def get_enabled_permanent(self, service, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()

        if service in fw_settings.getServices():
            return True
        else:
            return False

    def set_enabled_immediate(self, service, timeout):
        self.fw.addService(self.zone, service, timeout)

    def set_enabled_permanent(self, service, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addService(service)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, service, timeout):
        self.fw.removeService(self.zone, service)

    def set_disabled_permanent(self, service, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeService(service)
        self.update_fw_settings(fw_zone, fw_settings)


class ProtocolTransaction(FirewallTransaction):
    """
    ProtocolTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(ProtocolTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, protocol, timeout):
        if protocol in self.fw.getProtocols(self.zone):
            return True
        else:
            return False

    def get_enabled_permanent(self, protocol, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()

        if protocol in fw_settings.getProtocols():
            return True
        else:
            return False

    def set_enabled_immediate(self, protocol, timeout):
        self.fw.addProtocol(self.zone, protocol, timeout)

    def set_enabled_permanent(self, protocol, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addProtocol(protocol)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, protocol, timeout):
        self.fw.removeProtocol(self.zone, protocol)

    def set_disabled_permanent(self, protocol, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeProtocol(protocol)
        self.update_fw_settings(fw_zone, fw_settings)


class ForwardTransaction(FirewallTransaction):
    """
    ForwardTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(ForwardTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

        self.enabled_msg = "Added forward to zone %s" % self.zone
        self.disabled_msg = "Removed forward from zone %s" % self.zone

    def get_enabled_immediate(self):
        if self.fw.queryForward(self.zone) is True:
            return True
        else:
            return False

    def get_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        if fw_settings.queryForward() is True:
            return True
        else:
            return False

    def set_enabled_immediate(self):
        self.fw.addForward(self.zone)

    def set_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setForward(True)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self):
        self.fw.removeForward(self.zone)

    def set_disabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setForward(False)
        self.update_fw_settings(fw_zone, fw_settings)


class MasqueradeTransaction(FirewallTransaction):
    """
    MasqueradeTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(MasqueradeTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

        self.enabled_msg = "Added masquerade to zone %s" % self.zone
        self.disabled_msg = "Removed masquerade from zone %s" % self.zone

    def get_enabled_immediate(self):
        if self.fw.queryMasquerade(self.zone) is True:
            return True
        else:
            return False

    def get_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        if fw_settings.getMasquerade() is True:
            return True
        else:
            return False

    def set_enabled_immediate(self):
        self.fw.addMasquerade(self.zone)

    def set_enabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setMasquerade(True)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self):
        self.fw.removeMasquerade(self.zone)

    def set_disabled_permanent(self):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setMasquerade(False)
        self.update_fw_settings(fw_zone, fw_settings)


class PortTransaction(FirewallTransaction):
    """
    PortTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(PortTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, port, protocol, timeout):
        if self.fw_offline:
            dummy, fw_settings = self.get_fw_zone_settings()
            return fw_settings.queryPort(port=port, protocol=protocol)
        return self.fw.queryPort(zone=self.zone, port=port, protocol=protocol)

    def get_enabled_permanent(self, port, protocol, timeout):
        dummy, fw_settings = self.get_fw_zone_settings()
        return fw_settings.queryPort(port=port, protocol=protocol)

    def set_enabled_immediate(self, port, protocol, timeout):
        self.fw.addPort(self.zone, port, protocol, timeout)

    def set_enabled_permanent(self, port, protocol, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addPort(port, protocol)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, port, protocol, timeout):
        self.fw.removePort(self.zone, port, protocol)

    def set_disabled_permanent(self, port, protocol, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removePort(port, protocol)
        self.update_fw_settings(fw_zone, fw_settings)


class InterfaceTransaction(FirewallTransaction):
    """
    InterfaceTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(InterfaceTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

        self.enabled_msg = "Changed %s to zone %s" % \
            (self.action_args[0], self.zone)

        self.disabled_msg = "Removed %s from zone %s" % \
            (self.action_args[0], self.zone)

    def get_enabled_immediate(self, interface):
        if self.fw_offline:
            fw_zone, fw_settings = self.get_fw_zone_settings()
            interface_list = fw_settings.getInterfaces()
        else:
            interface_list = self.fw.getInterfaces(self.zone)
        if interface in interface_list:
            return True
        else:
            return False

    def get_enabled_permanent(self, interface):
        fw_zone, fw_settings = self.get_fw_zone_settings()

        if interface in fw_settings.getInterfaces():
            return True
        else:
            return False

    def set_enabled_immediate(self, interface):
        self.fw.changeZoneOfInterface(self.zone, interface)

    def set_enabled_permanent(self, interface):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        if self.fw_offline:
            iface_zone_objs = []
            for zone in self.fw.config.get_zones():
                old_zone_obj = self.fw.config.get_zone(zone)
                if interface in old_zone_obj.interfaces:
                    iface_zone_objs.append(old_zone_obj)

            if len(iface_zone_objs) > 1:
                # Even it shouldn't happen, it's actually possible that
                # the same interface is in several zone XML files
                self.module.fail_json(
                    msg='ERROR: interface {0} is in {1} zone XML file, can only be in one'.format(
                        interface,
                        len(iface_zone_objs)
                    )
                )
            elif len(iface_zone_objs) == 1 and iface_zone_objs[0].name != self.zone:
                old_zone_obj = iface_zone_objs[0]
                old_zone_config = self.fw.config.get_zone_config(old_zone_obj)
                old_zone_settings = FirewallClientZoneSettings(list(old_zone_config))
                old_zone_settings.removeInterface(interface)    # remove from old
                self.fw.config.set_zone_config(
                    old_zone_obj,
                    old_zone_settings.settings
                )
            fw_settings.addInterface(interface)             # add to new
            self.fw.config.set_zone_config(fw_zone, fw_settings.settings)
        else:
            old_zone_name = self.fw.config().getZoneOfInterface(interface)
            if old_zone_name != self.zone:
                if old_zone_name:
                    old_zone_obj = self.fw.config().getZoneByName(old_zone_name)
                    old_zone_settings = old_zone_obj.getSettings()
                    old_zone_settings.removeInterface(interface)  # remove from old
                    old_zone_obj.update(old_zone_settings)
                fw_settings.addInterface(interface)              # add to new
                fw_zone.update(fw_settings)

    def set_disabled_immediate(self, interface):
        self.fw.removeInterface(self.zone, interface)

    def set_disabled_permanent(self, interface):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeInterface(interface)
        self.update_fw_settings(fw_zone, fw_settings)


class RichRuleTransaction(FirewallTransaction):
    """
    RichRuleTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(RichRuleTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, rule, timeout):
        # Convert the rule string to standard format
        # before checking whether it is present
        rule = str(Rich_Rule(rule_str=rule))
        if rule in self.fw.getRichRules(self.zone):
            return True
        else:
            return False

    def get_enabled_permanent(self, rule, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        # Convert the rule string to standard format
        # before checking whether it is present
        rule = str(Rich_Rule(rule_str=rule))
        if rule in fw_settings.getRichRules():
            return True
        else:
            return False

    def set_enabled_immediate(self, rule, timeout):
        self.fw.addRichRule(self.zone, rule, timeout)

    def set_enabled_permanent(self, rule, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addRichRule(rule)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, rule, timeout):
        self.fw.removeRichRule(self.zone, rule)

    def set_disabled_permanent(self, rule, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeRichRule(rule)
        self.update_fw_settings(fw_zone, fw_settings)


class SourceTransaction(FirewallTransaction):
    """
    SourceTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(SourceTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

        self.enabled_msg = "Added %s to zone %s" % \
            (self.action_args[0], self.zone)

        self.disabled_msg = "Removed %s from zone %s" % \
            (self.action_args[0], self.zone)

    def get_enabled_immediate(self, source):
        if source in self.fw.getSources(self.zone):
            return True
        else:
            return False

    def get_enabled_permanent(self, source):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        if source in fw_settings.getSources():
            return True
        else:
            return False

    def set_enabled_immediate(self, source):
        self.fw.addSource(self.zone, source)

    def set_enabled_permanent(self, source):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addSource(source)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, source):
        self.fw.removeSource(self.zone, source)

    def set_disabled_permanent(self, source):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeSource(source)
        self.update_fw_settings(fw_zone, fw_settings)


class ZoneTargetTransaction(FirewallTransaction):
    """
    ZoneTargetTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None,
                 permanent=True, immediate=False, enabled_values=None, disabled_values=None):
        super(ZoneTargetTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone,
            permanent=permanent, immediate=immediate,
            enabled_values=enabled_values or ["present", "enabled"],
            disabled_values=disabled_values or ["absent", "disabled"])

        self.enabled_msg = "Set zone %s target to %s" % \
            (self.zone, action_args[0])

        self.disabled_msg = "Reset zone %s target to default" % \
            (self.zone)

        self.tx_not_permanent_error_msg = "Zone operations must be permanent. " \
            "Make sure you didn't set the 'permanent' flag to 'false' or the 'immediate' flag to 'true'."

    def get_enabled_immediate(self, target):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def get_enabled_permanent(self, target):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        current_target = fw_settings.getTarget()
        return (current_target == target)

    def set_enabled_immediate(self, target):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def set_enabled_permanent(self, target):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setTarget(target)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, target):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def set_disabled_permanent(self, target):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.setTarget("default")
        self.update_fw_settings(fw_zone, fw_settings)


class ZoneTransaction(FirewallTransaction):
    """
    ZoneTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None,
                 permanent=True, immediate=False, enabled_values=None, disabled_values=None):
        super(ZoneTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone,
            permanent=permanent, immediate=immediate,
            enabled_values=enabled_values or ["present"],
            disabled_values=disabled_values or ["absent"])

        self.enabled_msg = "Added zone %s" % \
            (self.zone)

        self.disabled_msg = "Removed zone %s" % \
            (self.zone)

        self.tx_not_permanent_error_msg = "Zone operations must be permanent. " \
            "Make sure you didn't set the 'permanent' flag to 'false' or the 'immediate' flag to 'true'."

    def get_enabled_immediate(self):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def get_enabled_permanent(self):
        if self.fw_offline:
            zones = self.fw.config.get_zones()
            zone_names = [self.fw.config.get_zone(z).name for z in zones]
        else:
            zones = self.fw.config().listZones()
            zone_names = [self.fw.config().getZone(z).get_property("name") for z in zones]
        return self.zone in zone_names

    def set_enabled_immediate(self):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def set_enabled_permanent(self):
        if self.fw_offline:
            self.fw.config.new_zone(self.zone, FirewallClientZoneSettings().settings)
        else:
            self.fw.config().addZone(self.zone, FirewallClientZoneSettings())

    def set_disabled_immediate(self):
        self.module.fail_json(msg=self.tx_not_permanent_error_msg)

    def set_disabled_permanent(self):
        if self.fw_offline:
            zone = self.fw.config.get_zone(self.zone)
            self.fw.config.remove_zone(zone)
        else:
            zone_obj = self.fw.config().getZoneByName(self.zone)
            zone_obj.remove()


class ForwardPortTransaction(FirewallTransaction):
    """
    ForwardPortTransaction
    """

    def __init__(self, module, action_args=None, zone=None, desired_state=None, permanent=False, immediate=False):
        super(ForwardPortTransaction, self).__init__(
            module, action_args=action_args, desired_state=desired_state, zone=zone, permanent=permanent, immediate=immediate
        )

    def get_enabled_immediate(self, port, proto, toport, toaddr, timeout):
        if self.fw_offline:
            dummy, fw_settings = self.get_fw_zone_settings()
            return fw_settings.queryForwardPort(port=port, protocol=proto, to_port=toport, to_addr=toaddr)
        return self.fw.queryForwardPort(zone=self.zone, port=port, protocol=proto, toport=toport, toaddr=toaddr)

    def get_enabled_permanent(self, port, proto, toport, toaddr, timeout):
        dummy, fw_settings = self.get_fw_zone_settings()
        return fw_settings.queryForwardPort(port=port, protocol=proto, to_port=toport, to_addr=toaddr)

    def set_enabled_immediate(self, port, proto, toport, toaddr, timeout):
        self.fw.addForwardPort(self.zone, port, proto, toport, toaddr, timeout)

    def set_enabled_permanent(self, port, proto, toport, toaddr, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.addForwardPort(port, proto, toport, toaddr)
        self.update_fw_settings(fw_zone, fw_settings)

    def set_disabled_immediate(self, port, proto, toport, toaddr, timeout):
        self.fw.removeForwardPort(self.zone, port, proto, toport, toaddr)

    def set_disabled_permanent(self, port, proto, toport, toaddr, timeout):
        fw_zone, fw_settings = self.get_fw_zone_settings()
        fw_settings.removeForwardPort(port, proto, toport, toaddr)
        self.update_fw_settings(fw_zone, fw_settings)


def main():

    module = AnsibleModule(
        argument_spec=dict(
            icmp_block=dict(type='str'),
            icmp_block_inversion=dict(type='bool'),
            service=dict(type='str'),
            protocol=dict(type='str'),
            port=dict(type='str'),
            port_forward=dict(type='list', elements='dict'),
            rich_rule=dict(type='str'),
            zone=dict(type='str'),
            immediate=dict(type='bool', default=False),
            source=dict(type='str'),
            permanent=dict(type='bool', default=False),
            state=dict(type='str', required=True, choices=['absent', 'disabled', 'enabled', 'present']),
            timeout=dict(type='int', default=0),
            interface=dict(type='str'),
            forward=dict(type='bool'),
            masquerade=dict(type='bool'),
            offline=dict(type='bool', default=False),
            target=dict(type='str', choices=['default', 'ACCEPT', 'DROP', '%%REJECT%%']),
        ),
        supports_check_mode=True,
        required_by=dict(
            interface=('zone',),
            target=('zone',),
            source=('permanent',),
        ),
        mutually_exclusive=[
            ['icmp_block', 'icmp_block_inversion', 'service', 'protocol', 'port', 'port_forward', 'rich_rule',
             'interface', 'forward', 'masquerade', 'source', 'target']
        ],
    )

    permanent = module.params['permanent']
    desired_state = module.params['state']
    immediate = module.params['immediate']
    timeout = module.params['timeout']
    interface = module.params['interface']
    forward = module.params['forward']
    masquerade = module.params['masquerade']
    offline = module.params['offline']

    # Sanity checks
    FirewallTransaction.sanity_check(module)

    # `offline`, `immediate`, and `permanent` have a weird twisty relationship.
    if offline:
        # specifying offline without permanent makes no sense
        if not permanent:
            module.fail_json(msg='offline cannot be enabled unless permanent changes are allowed')

        # offline overrides immediate to false if firewalld is offline
        if fw_offline:
            immediate = False

    # immediate defaults to true if permanent is not enabled
    if not permanent and not immediate:
        immediate = True

    if immediate and fw_offline:
        module.fail_json(msg='firewall is not currently running, unable to perform immediate actions without a running firewall daemon')

    # Verify required params are provided
    changed = False
    msgs = []
    icmp_block = module.params['icmp_block']
    icmp_block_inversion = module.params['icmp_block_inversion']
    service = module.params['service']
    protocol = module.params['protocol']
    rich_rule = module.params['rich_rule']
    source = module.params['source']
    zone = module.params['zone']
    target = module.params['target']

    port = None
    if module.params['port'] is not None:
        if '/' in module.params['port']:
            port, port_protocol = module.params['port'].strip().split('/')
        else:
            port_protocol = None
        if not port_protocol:
            module.fail_json(msg='improper port format (missing protocol?)')
    else:
        port_protocol = None

    port_forward_toaddr = ''
    port_forward = None
    if module.params['port_forward'] is not None:
        if len(module.params['port_forward']) > 1:
            module.fail_json(msg='Only one port forward supported at a time')
        port_forward = module.params['port_forward'][0]
        if 'port' not in port_forward:
            module.fail_json(msg='port must be specified for port forward')
        if 'proto' not in port_forward:
            module.fail_json(msg='proto udp/tcp must be specified for port forward')
        if 'toport' not in port_forward:
            module.fail_json(msg='toport must be specified for port forward')
        if 'toaddr' in port_forward:
            port_forward_toaddr = port_forward['toaddr']

    modification = False
    if any([icmp_block, icmp_block_inversion, service, protocol, port, port_forward, rich_rule,
            interface, forward, masquerade, source, target]):
        modification = True
    if modification and desired_state in ['absent', 'present'] and target is None:
        module.fail_json(
            msg='absent and present state can only be used in zone level operations'
        )

    if icmp_block is not None:

        transaction = IcmpBlockTransaction(
            module,
            action_args=(icmp_block, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed icmp-block %s to %s" % (icmp_block, desired_state))

    if icmp_block_inversion is not None:
        expected_state = 'enabled' if (desired_state == 'enabled') == icmp_block_inversion else 'disabled'
        transaction = IcmpBlockInversionTransaction(
            module,
            action_args=(),
            zone=zone,
            desired_state=expected_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed icmp-block-inversion %s to %s" % (icmp_block_inversion, desired_state))

    if service is not None:

        transaction = ServiceTransaction(
            module,
            action_args=(service, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed service %s to %s" % (service, desired_state))

    if protocol is not None:

        transaction = ProtocolTransaction(
            module,
            action_args=(protocol, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed protocol %s to %s" % (protocol, desired_state))

    if source is not None:

        transaction = SourceTransaction(
            module,
            action_args=(source,),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs

    if port is not None:

        transaction = PortTransaction(
            module,
            action_args=(port, port_protocol, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append(
                "Changed port %s to %s" % (
                    "%s/%s" % (port, port_protocol), desired_state
                )
            )

    if port_forward is not None:
        transaction = ForwardPortTransaction(
            module,
            action_args=(str(port_forward['port']), port_forward['proto'],
                         str(port_forward['toport']), port_forward_toaddr, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append(
                "Changed port_forward %s to %s" % (
                    "port=%s:proto=%s:toport=%s:toaddr=%s" % (
                        port_forward['port'], port_forward['proto'],
                        port_forward['toport'], port_forward_toaddr
                    ), desired_state
                )
            )

    if rich_rule is not None:

        transaction = RichRuleTransaction(
            module,
            action_args=(rich_rule, timeout),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed rich_rule %s to %s" % (rich_rule, desired_state))

    if interface is not None:

        transaction = InterfaceTransaction(
            module,
            action_args=(interface,),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs

    if forward is not None:
        expected_state = 'enabled' if (desired_state == 'enabled') == forward else 'disabled'
        transaction = ForwardTransaction(
            module,
            action_args=(),
            zone=zone,
            desired_state=expected_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs

    if masquerade is not None:
        expected_state = 'enabled' if (desired_state == 'enabled') == masquerade else 'disabled'
        transaction = MasqueradeTransaction(
            module,
            action_args=(),
            zone=zone,
            desired_state=expected_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs

    if target is not None:

        transaction = ZoneTargetTransaction(
            module,
            action_args=(target,),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs

    ''' If there are no changes within the zone we are operating on the zone itself '''
    if not modification and desired_state in ['absent', 'present']:

        transaction = ZoneTransaction(
            module,
            action_args=(),
            zone=zone,
            desired_state=desired_state,
            permanent=permanent,
            immediate=immediate,
        )

        changed, transaction_msgs = transaction.run()
        msgs = msgs + transaction_msgs
        if changed is True:
            msgs.append("Changed zone %s to %s" % (zone, desired_state))

    if fw_offline:
        msgs.append("(offline operation: only on-disk configs were altered)")

    module.exit_json(changed=changed, msg=', '.join(msgs))


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2016, Hiroaki Nakamura <hnakamur@gmail.com>
# Copyright (c) 2020, Frank Dornheim <dornheim@posteo.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: lxd_profile
short_description: Manage LXD profiles
description:
  - Management of LXD profiles.
author: "Hiroaki Nakamura (@hnakamur)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - Name of a profile.
    required: true
    type: str
  project:
    description:
      - Project of a profile. See U(https://documentation.ubuntu.com/lxd/en/latest/projects/).
    type: str
    version_added: 4.8.0
  description:
    description:
      - Description of the profile.
    type: str
  config:
    description:
      - 'The config for the instance (for example V({"limits.memory": "4GB"})).'
      - See U(https://documentation.ubuntu.com/lxd/en/latest/api/#/profiles/profile_get).
      - If the profile already exists and its C(config) value in metadata obtained from GET /1.0/profiles/<name>
        U(https://documentation.ubuntu.com/lxd/en/latest/api/#/profiles/profile_get)
        are different, then this module tries to apply the configurations U(https://documentation.ubuntu.com/lxd/en/latest/api/#/profiles/profile_put).
      - Not all config values are supported to apply the existing profile. Maybe you need to delete and recreate a profile.
    type: dict
  devices:
    description:
      - 'The devices for the profile (for example V({"rootfs": {"path": "/dev/kvm", "type": "unix-char"})).'
      - See U(https://documentation.ubuntu.com/lxd/en/latest/api/#/profiles/profile_get).
    type: dict
  new_name:
    description:
      - A new name of a profile.
      - If this parameter is specified a profile is renamed to this name.
      - See U(https://documentation.ubuntu.com/lxd/en/latest/api/#/profiles/profile_post).
    type: str
  merge_profile:
    description:
      - Merge the configuration of the present profile with the new desired configuration, instead of replacing it.
    default: false
    type: bool
    version_added: 2.1.0
  state:
    choices:
      - present
      - absent
    description:
      - Define the state of a profile.
    default: present
    type: str
  url:
    description:
      - The unix domain socket path or the https URL for the LXD server.
    default: unix:/var/lib/lxd/unix.socket
    type: str
  snap_url:
    description:
      - The unix domain socket path when LXD is installed by snap package manager.
    default: unix:/var/snap/lxd/common/lxd/unix.socket
    type: str
  client_key:
    description:
      - The client certificate key file path.
      - If not specified, it defaults to C($HOME/.config/lxc/client.key).
    aliases: [key_file]
    type: path
  client_cert:
    description:
      - The client certificate file path.
      - If not specified, it defaults to C($HOME/.config/lxc/client.crt).
    aliases: [cert_file]
    type: path
  trust_password:
    description:
      - The client trusted password.
      - 'You need to set this password on the LXD server before running this module using the following command: C(lxc config
        set core.trust_password <some random password>). See U(https://www.stgraber.org/2016/04/18/lxd-api-direct-interaction/).'
      - If O(trust_password) is set, this module send a request for authentication before sending any requests.
    type: str
notes:
  - Profiles must have a unique name. If you attempt to create a profile with a name that already existed in the users namespace
    the module simply returns as "unchanged".
"""

EXAMPLES = r"""
# An example for creating a profile
- hosts: localhost
  connection: local
  tasks:
    - name: Create a profile
      community.general.lxd_profile:
        name: macvlan
        state: present
        config: {}
        description: my macvlan profile
        devices:
          eth0:
            nictype: macvlan
            parent: br0
            type: nic

# An example for creating a profile in project mytestproject
- hosts: localhost
  connection: local
  tasks:
    - name: Create a profile
      community.general.lxd_profile:
        name: testprofile
        project: mytestproject
        state: present
        config: {}
        description: test profile in project mytestproject
        devices: {}

# An example for creating a profile via http connection
- hosts: localhost
  connection: local
  tasks:
    - name: Create macvlan profile
      community.general.lxd_profile:
        url: https://127.0.0.1:8443
        # These client_cert and client_key values are equal to the default values.
        # client_cert: "{{ lookup('env', 'HOME') }}/.config/lxc/client.crt"
        # client_key: "{{ lookup('env', 'HOME') }}/.config/lxc/client.key"
        trust_password: mypassword
        name: macvlan
        state: present
        config: {}
        description: my macvlan profile
        devices:
          eth0:
            nictype: macvlan
            parent: br0
            type: nic

# An example for modify/merge a profile
- hosts: localhost
  connection: local
  tasks:
    - name: Merge a profile
      community.general.lxd_profile:
        merge_profile: true
        name: macvlan
        state: present
        config: {}
        description: my macvlan profile
        devices:
          eth0:
            nictype: macvlan
            parent: br0
            type: nic

# An example for deleting a profile
- hosts: localhost
  connection: local
  tasks:
    - name: Delete a profile
      community.general.lxd_profile:
        name: macvlan
        state: absent

# An example for renaming a profile
- hosts: localhost
  connection: local
  tasks:
    - name: Rename a profile
      community.general.lxd_profile:
        name: macvlan
        new_name: macvlan2
        state: present
"""

RETURN = r"""
old_state:
  description: The old state of the profile.
  returned: success
  type: str
  sample: "absent"
logs:
  description: The logs of requests and responses.
  returned: when ansible-playbook is invoked with -vvvv.
  type: list
  sample: "(too long to be placed here)"
actions:
  description: List of actions performed for the profile.
  returned: success
  type: list
  sample: ["create"]
"""

import os
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lxd import LXDClient, LXDClientException

# ANSIBLE_LXD_DEFAULT_URL is a default value of the lxd endpoint
ANSIBLE_LXD_DEFAULT_URL = "unix:/var/lib/lxd/unix.socket"

# PROFILE_STATES is a list for states supported
PROFILES_STATES = ["present", "absent"]

# CONFIG_PARAMS is a list of config attribute names.
CONFIG_PARAMS = ["config", "description", "devices"]


class LXDProfileManagement:
    def __init__(self, module):
        """Management of LXC profiles via Ansible.

        :param module: Processed Ansible Module.
        :type module: ``object``
        """
        self.module = module
        self.name = self.module.params["name"]
        self.project = self.module.params["project"]
        self._build_config()
        self.state = self.module.params["state"]
        self.new_name = self.module.params.get("new_name", None)

        self.key_file = self.module.params.get("client_key")
        if self.key_file is None:
            self.key_file = f"{os.environ['HOME']}/.config/lxc/client.key"
        self.cert_file = self.module.params.get("client_cert")
        if self.cert_file is None:
            self.cert_file = f"{os.environ['HOME']}/.config/lxc/client.crt"
        self.debug = self.module._verbosity >= 4

        try:
            if self.module.params["url"] != ANSIBLE_LXD_DEFAULT_URL:
                self.url = self.module.params["url"]
            elif os.path.exists(self.module.params["snap_url"].replace("unix:", "")):
                self.url = self.module.params["snap_url"]
            else:
                self.url = self.module.params["url"]
        except Exception as e:
            self.module.fail_json(msg=e.msg)

        try:
            self.client = LXDClient(self.url, key_file=self.key_file, cert_file=self.cert_file, debug=self.debug)
        except LXDClientException as e:
            self.module.fail_json(msg=e.msg)
        self.trust_password = self.module.params.get("trust_password", None)
        self.actions = []

    def _build_config(self):
        self.config = {}
        for attr in CONFIG_PARAMS:
            param_val = self.module.params.get(attr, None)
            if param_val is not None:
                self.config[attr] = param_val

    def _get_profile_json(self):
        url = f"/1.0/profiles/{self.name}"
        if self.project:
            url = f"{url}?{urlencode(dict(project=self.project))}"
        return self.client.do("GET", url, ok_error_codes=[404])

    @staticmethod
    def _profile_json_to_module_state(resp_json):
        if resp_json["type"] == "error":
            return "absent"
        return "present"

    def _update_profile(self):
        if self.state == "present":
            if self.old_state == "absent":
                if self.new_name is None:
                    self._create_profile()
                else:
                    self.module.fail_json(
                        msg="new_name must not be set when the profile does not exist and the state is present",
                        changed=False,
                    )
            else:
                if self.new_name is not None and self.new_name != self.name:
                    self._rename_profile()
                if self._needs_to_apply_profile_configs():
                    self._apply_profile_configs()
        elif self.state == "absent":
            if self.old_state == "present":
                if self.new_name is None:
                    self._delete_profile()
                else:
                    self.module.fail_json(
                        msg="new_name must not be set when the profile exists and the specified state is absent",
                        changed=False,
                    )

    def _create_profile(self):
        url = "/1.0/profiles"
        if self.project:
            url = f"{url}?{urlencode(dict(project=self.project))}"
        config = self.config.copy()
        config["name"] = self.name
        self.client.do("POST", url, config)
        self.actions.append("create")

    def _rename_profile(self):
        url = f"/1.0/profiles/{self.name}"
        if self.project:
            url = f"{url}?{urlencode(dict(project=self.project))}"
        config = {"name": self.new_name}
        self.client.do("POST", url, config)
        self.actions.append("rename")
        self.name = self.new_name

    def _needs_to_change_profile_config(self, key):
        if key not in self.config:
            return False
        old_configs = self.old_profile_json["metadata"].get(key, None)
        return self.config[key] != old_configs

    def _needs_to_apply_profile_configs(self):
        return (
            self._needs_to_change_profile_config("config")
            or self._needs_to_change_profile_config("description")
            or self._needs_to_change_profile_config("devices")
        )

    def _merge_dicts(self, source, destination):
        """Merge Dictionaries

        Get a list of filehandle numbers from logger to be handed to
        DaemonContext.files_preserve

        Args:
            dict(source): source dict
            dict(destination): destination dict
        Kwargs:
            None
        Raises:
            None
        Returns:
            dict(destination): merged dict"""
        for key, value in source.items():
            if isinstance(value, dict):
                # get node or create one
                node = destination.setdefault(key, {})
                self._merge_dicts(value, node)
            else:
                destination[key] = value
        return destination

    def _merge_config(self, config):
        """merge profile

        Merge Configuration of the present profile and the new desired configitems

        Args:
            dict(config): Dict with the old config in 'metadata' and new config in 'config'
        Kwargs:
            None
        Raises:
            None
        Returns:
            dict(config): new config"""
        # merge or copy the sections from the existing profile to 'config'
        for item in ["config", "description", "devices", "name", "used_by"]:
            if item in config:
                config[item] = self._merge_dicts(config["metadata"][item], config[item])
            else:
                config[item] = config["metadata"][item]
        # merge or copy the sections from the ansible-task to 'config'
        return self._merge_dicts(self.config, config)

    def _generate_new_config(self, config):
        """rebuild profile

        Rebuild the Profile by the configuration provided in the play.
        Existing configurations are discarded.

        This is the default behavior.

        Args:
            dict(config): Dict with the old config in 'metadata' and new config in 'config'
        Kwargs:
            None
        Raises:
            None
        Returns:
            dict(config): new config"""
        for k, v in self.config.items():
            config[k] = v
        return config

    def _apply_profile_configs(self):
        """Selection of the procedure: rebuild or merge

        The standard behavior is that all information not contained
        in the play is discarded.

        If "merge_profile" is provides in the play and "True", then existing
        configurations from the profile and new ones defined are merged.

        Args:
            None
        Kwargs:
            None
        Raises:
            None
        Returns:
            None"""
        config = self.old_profile_json.copy()
        if self.module.params["merge_profile"]:
            config = self._merge_config(config)
        else:
            config = self._generate_new_config(config)

        # upload config to lxd
        url = f"/1.0/profiles/{self.name}"
        if self.project:
            url = f"{url}?{urlencode(dict(project=self.project))}"
        self.client.do("PUT", url, config)
        self.actions.append("apply_profile_configs")

    def _delete_profile(self):
        url = f"/1.0/profiles/{self.name}"
        if self.project:
            url = f"{url}?{urlencode(dict(project=self.project))}"
        self.client.do("DELETE", url)
        self.actions.append("delete")

    def run(self):
        """Run the main method."""

        try:
            if self.trust_password is not None:
                self.client.authenticate(self.trust_password)

            self.old_profile_json = self._get_profile_json()
            self.old_state = self._profile_json_to_module_state(self.old_profile_json)
            self._update_profile()

            state_changed = len(self.actions) > 0
            result_json = {"changed": state_changed, "old_state": self.old_state, "actions": self.actions}
            if self.client.debug:
                result_json["logs"] = self.client.logs
            self.module.exit_json(**result_json)
        except LXDClientException as e:
            state_changed = len(self.actions) > 0
            fail_params = {"msg": e.msg, "changed": state_changed, "actions": self.actions}
            if self.client.debug:
                fail_params["logs"] = e.kwargs["logs"]
            self.module.fail_json(**fail_params)


def main():
    """Ansible Main module."""

    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            project=dict(
                type="str",
            ),
            new_name=dict(
                type="str",
            ),
            config=dict(
                type="dict",
            ),
            description=dict(
                type="str",
            ),
            devices=dict(
                type="dict",
            ),
            merge_profile=dict(type="bool", default=False),
            state=dict(choices=PROFILES_STATES, default="present"),
            url=dict(type="str", default=ANSIBLE_LXD_DEFAULT_URL),
            snap_url=dict(type="str", default="unix:/var/snap/lxd/common/lxd/unix.socket"),
            client_key=dict(type="path", aliases=["key_file"]),
            client_cert=dict(type="path", aliases=["cert_file"]),
            trust_password=dict(type="str", no_log=True),
        ),
        supports_check_mode=False,
    )

    lxd_manage = LXDProfileManagement(module=module)
    lxd_manage.run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = r"""
---
module: s3_object_info
version_added: 5.0.0
short_description: Gather information about objects in S3
description:
    - Describes objects in S3.
    - Compatible with AWS, DigitalOcean, Ceph, Walrus, FakeS3 and StorageGRID (only supports list_keys currently).
    - When using non-AWS services, O(endpoint_url) should be specified.
author:
  - Mandar Vijay Kulkarni (@mandar242)
options:
  bucket_name:
    description:
      - The name of the bucket that contains the object.
    required: true
    type: str
  object_name:
    description:
      - The name of the object.
      - If not specified, a list of all objects in the specified bucket will be returned.
      - Mutually exclusive with O(prefix).
    required: false
    type: str
  endpoint_url:
    description:
      - S3 URL endpoint for usage with Ceph, Eucalyptus and fakes3 etc. Otherwise assumes AWS.
    type: str
  dualstack:
    description:
      - Enables Amazon S3 Dual-Stack Endpoints, allowing S3 communications using both IPv4 and IPv6.
      - Mutually exclusive with O(endpoint_url).
    type: bool
    default: false
  ceph:
    description:
      - Enable API compatibility with Ceph RGW.
      - It takes into account the S3 API subset working with Ceph in order to provide the same module
        behaviour where possible.
      - Requires O(endpoint_url) if O(ceph=true).
    aliases: ['rgw']
    default: false
    type: bool
  object_details:
    description:
      - Retrieve requested S3 object detailed information.
    required: false
    type: dict
    suboptions:
      object_acl:
        description:
          - Retreive S3 object ACL.
        required: false
        type: bool
        default: false
      object_legal_hold:
        description:
          - Retreive S3 object legal_hold.
        required: false
        type: bool
        default: false
      object_lock_configuration:
        description:
          - Retreive S3 object lock_configuration.
        required: false
        type: bool
        default: false
      object_retention:
        description:
          - Retreive S3 object retention.
        required: false
        type: bool
        default: false
      object_tagging:
        description:
          - Retreive S3 object Tags.
        required: false
        type: bool
        default: false
      object_attributes:
        description:
          - Retreive S3 object attributes.
        required: false
        type: bool
        default: false
      attributes_list:
        description:
          - The fields/details that should be returned.
          - Required when O(object_details.object_attributes=true).
        type: list
        elements: str
        choices: ['ETag', 'Checksum', 'ObjectParts', 'StorageClass', 'ObjectSize']
  marker:
    description:
      - Specifies the Object key to start with.  Object keys are returned in alphabetical order, starting with key
        after the marker in order.
    type: str
    version_added: 9.0.0
  max_keys:
    description:
      - Max number of results to return.  Set this if you want to retrieve only partial results.
    type: int
    version_added: 9.0.0
  prefix:
    description:
      - Limits the response to keys that begin with the specified prefix.
      - Mutually exclusive with O(object_name).
    type: str
    version_added: 9.2.0
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Retrieve a list of objects in S3 bucket
  amazon.aws.s3_object_info:
    bucket_name: MyTestBucket

- name: Retrieve a list of objects in Ceph RGW S3
  amazon.aws.s3_object_info:
    bucket_name: MyTestBucket
    ceph: true
    endpoint_url: "http://localhost:8000"

- name: Retrieve object metadata without object itself
  amazon.aws.s3_object_info:
    bucket_name: MyTestBucket
    object_name: MyTestObjectKey

- name: Retrieve detailed S3 information for all objects in the bucket
  amazon.aws.s3_object_info:
    bucket_name: MyTestBucket
    object_details:
      object_acl: true
      object_attributes: true
      attributes_list:
        - ETag
        - ObjectSize
        - StorageClass

- name: Retrieve detailed S3 object information
  amazon.aws.s3_object_info:
    bucket_name: MyTestBucket
    object_name: MyTestObjectKey
    object_details:
      object_acl: true
      object_tagging: true
      object_legal_hold: true
      object_attributes: true
      attributes_list:
        - ETag
        - ObjectSize

- name: Retrieve keys that begin with the prefix /my/desired/
  amazon.aws.s3_object_info:
    bucket: mybucket
    prefix: /my/desired/
"""

RETURN = r"""
s3_keys:
  description: List of object keys.
  returned: when only O(bucket_name) is specified and O(object_name), O(object_details) are not specified.
  type: list
  elements: str
  sample:
  - prefix1/
  - prefix1/key1
  - prefix1/key2
object_info:
    description: S3 object details.
    returned: when O(bucket_name) and O(object_name) are specified.
    type: list
    elements: dict
    contains:
        object_data:
            description: A dict containing the metadata of S3 object.
            returned: when O(bucket_name) and O(object_name) are specified but O(object_details) is not specified.
            type: dict
            elements: str
            contains:
                accept_ranges:
                    description: Indicates that a range of bytes was specified.
                    returned: always
                    type: str
                content_length:
                    description: Size of the body (object data) in bytes.
                    returned: always
                    type: int
                content_type:
                    description: A standard MIME type describing the format of the object data.
                    returned: always
                    type: str
                e_tag:
                    description: A opaque identifier assigned by a web server to a specific version of a resource found at a URL.
                    returned: always
                    type: str
                last_modified:
                    description: Creation date of the object.
                    returned: always
                    type: str
                metadata:
                    description: A map of metadata to store with the object in S3.
                    returned: always
                    type: dict
                server_side_encryption:
                    description: The server-side encryption algorithm used when storing this object in Amazon S3.
                    returned: always
                    type: str
                tag_count:
                    description: The number of tags, if any, on the object.
                    returned: always
                    type: int
        object_acl:
            description: Access control list (ACL) of an object.
            returned: when O(object_details.object_acl=true).
            type: complex
            contains:
                owner:
                    description: Bucket owner's display ID and name.
                    returned: always
                    type: complex
                    contains:
                        id:
                            description: Bucket owner's ID.
                            returned: always
                            type: str
                            sample: "xxxxxxxxxxxxxxxxxxxxx"
                        display_name:
                            description: Bucket owner's display name.
                            returned: always
                            type: str
                            sample: 'abcd'
                grants:
                    description: A list of grants.
                    returned: always
                    type: complex
                    contains:
                        grantee:
                            description: The entity being granted permissions.
                            returned: always
                            type: complex
                            contains:
                                id:
                                    description: The canonical user ID of the grantee.
                                    returned: always
                                    type: str
                                    sample: "xxxxxxxxxxxxxxxxxxx"
                                type:
                                    description: type of grantee.
                                    returned: always
                                    type: str
                                    sample: "CanonicalUser"
                        permission:
                            description: Specifies the permission given to the grantee.
                            returned: always
                            type: str
                            sample: "FULL CONTROL"
        object_legal_hold:
            description: Object's current legal hold status
            returned: when O(object_details.object_legal_hold=true) and object legal hold is set on the bucket.
            type: complex
            contains:
                legal_hold:
                    description: The current legal hold status for the specified object.
                    returned: always
                    type: complex
                    contains:
                        status:
                            description: Indicates whether the specified object has a legal hold in place.
                            returned: always
                            type: str
                            sample: "ON"
        object_lock_configuration:
            description: Object Lock configuration for a bucket.
            returned: when O(object_details.object_lock_configuration=true) and object lock configuration is set on the bucket.
            type: complex
            contains:
                object_lock_enabled:
                    description: Indicates whether this bucket has an Object Lock configuration enabled.
                    returned: always
                    type: str
                rule:
                    description: Specifies the Object Lock rule for the specified object.
                    returned: always
                    type: complex
                    contains:
                        default_retention:
                            description: The default Object Lock retention mode and period that you want to apply to new objects placed in the specified bucket.
                            returned: always
                            type: complex
                            contains:
                                mode:
                                    description:
                                      - The default Object Lock retention mode you want to apply to new objects placed in the specified bucket.
                                      - Must be used with either Days or Years.
                                    returned: always
                                    type: str
                                days:
                                    description: The number of days that you want to specify for the default retention period.
                                    returned: always
                                    type: int
                                years:
                                    description: The number of years that you want to specify for the default retention period.
                                    returned: always
                                    type: int
        object_retention:
            description: Object's retention settings.
            returned: when O(object_details.object_retention=true) and object retention is set on the bucket.
            type: complex
            contains:
                retention:
                    description: The container element for an object's retention settings.
                    returned: always
                    type: complex
                    contains:
                        mode:
                            description: Indicates the Retention mode for the specified object.
                            returned: always
                            type: str
                        retain_until_date:
                            description: The date on which this Object Lock Retention will expire.
                            returned: always
                            type: str
        object_tagging:
            description: The tag-set of an object
            returned: when O(object_details.object_tagging=true).
            type: dict
        object_attributes:
            description: Object attributes.
            returned: when O(object_details.object_attributes=true).
            type: complex
            contains:
                etag:
                    description: An ETag is an opaque identifier assigned by a web server to a specific version of a resource found at a URL.
                    returned: always
                    type: str
                    sample: "8fa34xxxxxxxxxxxxxxxxxxxxx35c6f3b"
                last_modified:
                    description: The creation date of the object.
                    returned: always
                    type: str
                    sample: "2022-08-10T01:11:03+00:00"
                object_size:
                    description: The size of the object in bytes.
                    returned: alwayS
                    type: int
                    sample: 819
                checksum:
                    description: The checksum or digest of the object.
                    returned: always
                    type: complex
                    contains:
                        checksum_crc32:
                            description: The base64-encoded, 32-bit CRC32 checksum of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_crc32c:
                            description: The base64-encoded, 32-bit CRC32C checksum of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_sha1:
                            description: The base64-encoded, 160-bit SHA-1 digest of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_sha256:
                            description: The base64-encoded, 256-bit SHA-256 digest of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                object_parts:
                    description: A collection of parts associated with a multipart upload.
                    returned: always
                    type: complex
                    contains:
                        total_parts_count:
                            description: The total number of parts.
                            returned: always
                            type: int
                        part_number_marker:
                            description: The marker for the current part.
                            returned: always
                            type: int
                        next_part_number_marker:
                            description:
                              - When a list is truncated, this element specifies the last part in the list
                              - As well as the value to use for the PartNumberMarker request parameter in a subsequent request.
                            returned: always
                            type: int
                        max_parts:
                            description: The maximum number of parts allowed in the response.
                            returned: always
                            type: int
                        is_truncated:
                            description: Indicates whether the returned list of parts is truncated.
                            returned: always
                            type: bool
                storage_class:
                    description: The storage class information of the object.
                    returned: always
                    type: str
                    sample: "STANDARD"
                parts:
                    description: A container for elements related to an individual part.
                    returned: always
                    type: complex
                    contains:
                        part_number:
                            description: The part number identifying the part. This value is a positive integer between 1 and 10,000.
                            returned: always
                            type: int
                        size:
                            description: The size of the uploaded part in bytes.
                            returned: always
                            type: int
                        checksum_crc32:
                            description: The base64-encoded, 32-bit CRC32 checksum of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_crc32c:
                            description: The base64-encoded, 32-bit CRC32C checksum of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_sha1:
                            description: The base64-encoded, 160-bit SHA-1 digest of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
                        checksum_sha256:
                            description: The base64-encoded, 256-bit SHA-256 digest of the object.
                            returned: if it was upload with the object.
                            type: str
                            sample: "xxxxxxxxxxxx"
"""

import typing

if typing.TYPE_CHECKING:
    from typing import Dict
    from typing import List

    from ansible_collections.amazon.aws.plugins.module_utils.botocore import ClientType

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.s3 import AnsibleS3Error
from ansible_collections.amazon.aws.plugins.module_utils.s3 import AnsibleS3SupportError
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_acl
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_attributes
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_legal_hold
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_lock_configuration
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_retention
from ansible_collections.amazon.aws.plugins.module_utils.s3 import get_s3_object_tagging
from ansible_collections.amazon.aws.plugins.module_utils.s3 import head_s3_object
from ansible_collections.amazon.aws.plugins.module_utils.s3 import list_bucket_object_keys
from ansible_collections.amazon.aws.plugins.module_utils.s3 import s3_bucket_exists
from ansible_collections.amazon.aws.plugins.module_utils.s3 import s3_extra_params
from ansible_collections.amazon.aws.plugins.module_utils.s3 import s3_object_exists


def describe_s3_object_acl(connection: ClientType, bucket_name: str, object_name: str) -> Dict:
    """Get object ACL."""
    try:
        acl_info = get_s3_object_acl(connection, bucket_name, object_name)
        if acl_info:
            return camel_dict_to_snake_dict(acl_info)
        return {}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {}


def describe_s3_object_attributes(
    connection: ClientType, module: AnsibleAWSModule, bucket_name: str, object_name: str
) -> Dict:
    """Get object attributes."""
    try:
        attributes_list = module.params.get("object_details", {}).get("attributes_list", [])
        if not attributes_list:
            return {}

        attrs = get_s3_object_attributes(connection, bucket_name, object_name, object_attributes=attributes_list)
        if attrs:
            return camel_dict_to_snake_dict(attrs)
        return {}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {"msg": "Object attributes not found"}
    except AnsibleS3Error:
        return {"msg": "Object attributes not found"}


def describe_s3_object_legal_hold(connection: ClientType, bucket_name: str, object_name: str) -> Dict:
    """Get object legal hold status."""
    try:
        legal_hold = get_s3_object_legal_hold(connection, bucket_name, object_name)
        if legal_hold:
            return {"legal_hold": camel_dict_to_snake_dict(legal_hold)}
        return {}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {}


def describe_s3_object_lock_configuration(connection: ClientType, bucket_name: str) -> Dict:
    """Get bucket-level object lock configuration."""
    try:
        lock_config = get_s3_object_lock_configuration(connection, bucket_name)
        if lock_config:
            return camel_dict_to_snake_dict(lock_config)
        return {}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {}


def describe_s3_object_retention(connection: ClientType, bucket_name: str, object_name: str) -> Dict:
    """Get object retention settings."""
    try:
        retention = get_s3_object_retention(connection, bucket_name, object_name)
        if retention:
            return {"retention": camel_dict_to_snake_dict(retention)}
        return {}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {}


def describe_s3_object_tagging(connection: ClientType, bucket_name: str, object_name: str) -> Dict:
    """Get object tags."""
    try:
        return get_s3_object_tagging(connection, bucket_name, object_name)
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {}


def get_object_details(
    connection: ClientType, module: AnsibleAWSModule, bucket_name: str, object_name: str, requested_facts: Dict
) -> Dict:
    all_facts = {}

    # Remove non-requested facts
    requested_facts = {fact: value for fact, value in requested_facts.items() if value is True}

    all_facts["object_data"] = get_object(connection, bucket_name, object_name)["object_data"]

    # Below APIs do not return object_name, need to add it manually
    all_facts["object_name"] = object_name

    for key in requested_facts:
        if key == "object_acl":
            all_facts[key] = describe_s3_object_acl(connection, bucket_name, object_name)
        elif key == "object_attributes":
            all_facts[key] = describe_s3_object_attributes(connection, module, bucket_name, object_name)
        elif key == "object_legal_hold":
            all_facts[key] = describe_s3_object_legal_hold(connection, bucket_name, object_name)
        elif key == "object_lock_configuration":
            all_facts[key] = describe_s3_object_lock_configuration(connection, bucket_name)
        elif key == "object_retention":
            all_facts[key] = describe_s3_object_retention(connection, bucket_name, object_name)
        elif key == "object_tagging":
            all_facts[key] = describe_s3_object_tagging(connection, bucket_name, object_name)

    return all_facts


def get_object(connection: ClientType, bucket_name: str, object_name: str) -> Dict:
    """Get basic object metadata."""
    try:
        object_info = head_s3_object(connection, bucket_name, object_name)
        if object_info:
            object_info = camel_dict_to_snake_dict(object_info)
        return {"object_data": object_info}
    except AnsibleS3SupportError:
        # Silent failure for unsupported operations on S3 drop-ins
        return {"object_data": {}}
    except AnsibleS3Error:
        # Silent failure to maintain backward compatibility
        return {"object_data": {}}


def list_bucket_objects(connection: ClientType, module: AnsibleAWSModule, bucket_name: str) -> List[str]:
    keys = list_bucket_object_keys(
        connection,
        bucket=bucket_name,
        max_keys=module.params["max_keys"],
        start_after=module.params["marker"],
        prefix=module.params["prefix"],
    )
    return keys


def bucket_check(connection: ClientType, module: AnsibleAWSModule, bucket_name: str) -> None:
    """Check if bucket exists and is accessible."""
    if not s3_bucket_exists(connection, bucket_name):
        module.fail_json(msg=f"The bucket {bucket_name} does not exist.")


def object_check(connection: ClientType, module: AnsibleAWSModule, bucket_name: str, object_name: str) -> None:
    """Check if object exists and is accessible."""
    if not s3_object_exists(connection, bucket_name, object_name):
        module.fail_json(msg=f"The object {object_name} does not exist.")


def main() -> None:
    argument_spec = dict(
        object_details=dict(
            type="dict",
            options=dict(
                object_acl=dict(type="bool", default=False),
                object_legal_hold=dict(type="bool", default=False),
                object_lock_configuration=dict(type="bool", default=False),
                object_retention=dict(type="bool", default=False),
                object_tagging=dict(type="bool", default=False),
                object_attributes=dict(type="bool", default=False),
                attributes_list=dict(
                    type="list",
                    elements="str",
                    choices=["ETag", "Checksum", "ObjectParts", "StorageClass", "ObjectSize"],
                ),
            ),
            required_if=[
                ("object_attributes", True, ["attributes_list"]),
            ],
        ),
        bucket_name=dict(required=True, type="str"),
        object_name=dict(type="str"),
        dualstack=dict(default=False, type="bool"),
        ceph=dict(default=False, type="bool", aliases=["rgw"]),
        marker=dict(),
        max_keys=dict(type="int", no_log=False),
        prefix=dict(type="str", required=False),
    )

    required_if = [
        ["ceph", True, ["endpoint_url"]],
    ]

    mutually_exclusive = [
        ["object_name", "prefix"],
        ["dualstack", "endpoint_url"],
    ]
    mutually_exclusive = [["object_name", "prefix"], ["dualstack", "endpoint_url"]]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
    )

    bucket_name = module.params.get("bucket_name")
    object_name = module.params.get("object_name")
    requested_object_details = module.params.get("object_details")

    result = []
    extra_params = s3_extra_params(module.params)
    retry_decorator = AWSRetry.jittered_backoff()
    connection = module.client("s3", retry_decorator=retry_decorator, **extra_params)

    # check if specified bucket exists
    bucket_check(connection, module, bucket_name)
    # check if specified object exists
    if object_name:
        object_check(connection, module, bucket_name, object_name)

    if requested_object_details:
        if object_name:
            object_details = get_object_details(connection, module, bucket_name, object_name, requested_object_details)
            result.append(object_details)
        elif object_name is None:
            object_list = list_bucket_objects(connection, module, bucket_name)
            for bucket_object in object_list:
                result.append(
                    get_object_details(connection, module, bucket_name, bucket_object, requested_object_details)
                )

    elif not requested_object_details and object_name:
        # if specific details are not requested, return object metadata
        object_details = get_object(connection, bucket_name, object_name)
        result.append(object_details)
    else:
        # return list of all objects in a bucket if object name and object details not specified
        object_list = list_bucket_objects(connection, module, bucket_name)
        module.exit_json(s3_keys=object_list)

    module.exit_json(object_info=result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_ssids_bonjour_forwarding
short_description: Resource module for networks _wireless _ssids _bonjour _forwarding
description:
  - Manage operation update of the resource networks _wireless _ssids _bonjour _forwarding.
  - Update the bonjour forwarding setting and rules for the SSID.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  enabled:
    description: If true, Bonjour forwarding is enabled on this SSID.
    type: bool
  exception:
    description: Bonjour forwarding exception.
    suboptions:
      enabled:
        description: If true, Bonjour forwarding exception is enabled on this SSID.
          Exception is required to enable L2 isolation and Bonjour forwarding to work
          together.
        type: bool
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  rules:
    description: List of bonjour forwarding rules.
    elements: dict
    suboptions:
      description:
        description: A description for your Bonjour forwarding rule. Optional.
        type: str
      services:
        description: A list of Bonjour services. At least one service must be specified.
          Available services are 'All Services', 'AFP', 'AirPlay', 'Apple screen share',
          'BitTorrent', 'Chromecast', 'FTP', 'iChat', 'iTunes', 'Printers', 'Samba',
          'Scanners', 'Spotify' and 'SSH'.
        elements: str
        type: list
      vlanId:
        description: The ID of the service VLAN. Required.
        type: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessSsidBonjourForwarding
    description: Complete reference of the updateNetworkWirelessSsidBonjourForwarding
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ssid-bonjour-forwarding
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_ssid_bonjour_forwarding,
  - Paths used are
    put /networks/{networkId}/wireless/ssids/{number}/bonjourForwarding,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_wireless_ssids_bonjour_forwarding:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    enabled: true
    exception:
      enabled: true
    networkId: string
    number: string
    rules:
      - description: A simple bonjour rule
        services:
          - All Services
        vlanId: '1'
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enabled": true,
      "exception": {
        "enabled": true
      },
      "rules": [
        {
          "description": "string",
          "vlanId": "string",
          "services": [
            "string"
          ]
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_latency_stats_info
short_description: Information module for networks _wireless _latency _stats
description:
  - Get all networks _wireless _latency _stats.
  - Aggregated latency info for this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  t0:
    description:
      - >
        T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 180 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 7 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 7 days.
    type: float
  band:
    description:
      - >
        Band query parameter. Filter results by band (either '2.4', '5' or '6'). Note
        that data prior to February 2020 will not have band information.
    type: str
  ssid:
    description:
      - Ssid query parameter. Filter results by SSID.
    type: int
  apTag:
    description:
      - ApTag query parameter. Filter results by AP Tag.
    type: str
  vlan:
    description:
      - Vlan query parameter. Filter results by VLAN.
    type: int
  fields:
    description:
      - >
        Fields query parameter. Partial selection If present, this call will return
        only the selected fields of "rawDistribution", "avg". All fields will be returned
        by default. Selected fields must be entered as a comma separated string.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getNetworkWirelessLatencyStats
    description: Complete reference of the getNetworkWirelessLatencyStats API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-latency-stats
notes:
  - SDK Method used are
    wireless.Wireless.get_network_wireless_latency_stats,
  - Paths used are
    get /networks/{networkId}/wireless/latencyStats,
"""

EXAMPLES = r"""
- name: Get all networks _wireless _latency _stats
  cisco.meraki.networks_wireless_latency_stats_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    t0: string
    t1: string
    timespan: 0
    band: string
    ssid: 0
    apTag: string
    vlan: 0
    fields: string
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "backgroundTraffic": {
        "rawDistribution": {},
        "avg": 0
      },
      "bestEffortTraffic": {
        "rawDistribution": {},
        "avg": 0
      },
      "videoTraffic": {
        "rawDistribution": {},
        "avg": 0
      },
      "voiceTraffic": {
        "rawDistribution": {},
        "avg": 0
      }
    }
"""



# Based on local.py by Michael DeHaan <michael.dehaan@gmail.com>
# and chroot.py by  Maykel Moya <mmoya@speedyrails.com>
# Copyright (c) 2013, Michael Scherer <misc@zarb.org>
# Copyright (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Ansible Core Team
name: jail
short_description: Run tasks in jails
description:
  - Run commands or put/fetch files to an existing jail.
options:
  remote_addr:
    description:
      - Path to the jail.
    type: string
    default: inventory_hostname
    vars:
      - name: inventory_hostname
      - name: ansible_host
      - name: ansible_jail_host
  remote_user:
    description:
      - User to execute as inside the jail.
    type: string
    vars:
      - name: ansible_user
      - name: ansible_jail_user
"""

import os
import os.path
import subprocess
import traceback
from shlex import quote as shlex_quote

from ansible.errors import AnsibleError
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_bytes, to_native, to_text
from ansible.plugins.connection import BUFSIZE, ConnectionBase
from ansible.utils.display import Display

display = Display()


class Connection(ConnectionBase):
    """Local BSD Jail based connections"""

    modified_jailname_key = "conn_jail_name"

    transport = "community.general.jail"
    # Pipelining may work.  Someone needs to test by setting this to True and
    # having pipelining=True in their ansible.cfg
    has_pipelining = True
    has_tty = False

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super().__init__(play_context, new_stdin, *args, **kwargs)

        self.jail = self._play_context.remote_addr
        if self.modified_jailname_key in kwargs:
            self.jail = kwargs[self.modified_jailname_key]

        if os.geteuid() != 0:
            raise AnsibleError("jail connection requires running as root")

        self.jls_cmd = self._search_executable("jls")
        self.jexec_cmd = self._search_executable("jexec")

        if self.jail not in self.list_jails():
            raise AnsibleError(f"incorrect jail name {self.jail}")

    @staticmethod
    def _search_executable(executable):
        try:
            return get_bin_path(executable)
        except ValueError as e:
            raise AnsibleError(f"{executable} command not found in PATH") from e

    def list_jails(self):
        p = subprocess.Popen(
            [self.jls_cmd, "-q", "name"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

        stdout, stderr = p.communicate()

        return to_text(stdout, errors="surrogate_or_strict").split()

    def _connect(self):
        """connect to the jail; nothing to do here"""
        super()._connect()
        if not self._connected:
            display.vvv(f"ESTABLISH JAIL CONNECTION FOR USER: {self._play_context.remote_user}", host=self.jail)
            self._connected = True

    def _buffered_exec_command(self, cmd, stdin=subprocess.PIPE):
        """run a command on the jail.  This is only needed for implementing
        put_file() get_file() so that we don't have to read the whole file
        into memory.

        compared to exec_command() it looses some niceties like being able to
        return the process's exit code immediately.
        """

        local_cmd = [self.jexec_cmd]
        set_env = ""

        if self._play_context.remote_user is not None:
            local_cmd += ["-U", self._play_context.remote_user]
            # update HOME since -U does not update the jail environment
            set_env = f"HOME=~{self._play_context.remote_user} "

        local_cmd += [self.jail, self._play_context.executable, "-c", set_env + cmd]

        display.vvv(f"EXEC {local_cmd}", host=self.jail)
        local_cmd = [to_bytes(i, errors="surrogate_or_strict") for i in local_cmd]
        p = subprocess.Popen(local_cmd, shell=False, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        return p

    def exec_command(self, cmd, in_data=None, sudoable=False):
        """run a command on the jail"""
        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)

        p = self._buffered_exec_command(cmd)

        stdout, stderr = p.communicate(in_data)
        return p.returncode, stdout, stderr

    @staticmethod
    def _prefix_login_path(remote_path):
        """Make sure that we put files into a standard path

        If a path is relative, then we need to choose where to put it.
        ssh chooses $HOME but we aren't guaranteed that a home dir will
        exist in any given chroot.  So for now we're choosing "/" instead.
        This also happens to be the former default.

        Can revisit using $HOME instead if it is a problem
        """
        if not remote_path.startswith(os.path.sep):
            remote_path = os.path.join(os.path.sep, remote_path)
        return os.path.normpath(remote_path)

    def put_file(self, in_path, out_path):
        """transfer a file from local to jail"""
        super().put_file(in_path, out_path)
        display.vvv(f"PUT {in_path} TO {out_path}", host=self.jail)

        out_path = shlex_quote(self._prefix_login_path(out_path))
        try:
            with open(to_bytes(in_path, errors="surrogate_or_strict"), "rb") as in_file:
                if not os.fstat(in_file.fileno()).st_size:
                    count = " count=0"
                else:
                    count = ""
                try:
                    p = self._buffered_exec_command(f"dd of={out_path} bs={BUFSIZE}{count}", stdin=in_file)
                except OSError as e:
                    raise AnsibleError("jail connection requires dd command in the jail") from e
                try:
                    stdout, stderr = p.communicate()
                except Exception as e:
                    traceback.print_exc()
                    raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
                if p.returncode != 0:
                    raise AnsibleError(
                        f"failed to transfer file {in_path} to {out_path}:\n{to_native(stdout)}\n{to_native(stderr)}"
                    )
        except OSError as e:
            raise AnsibleError(f"file or module does not exist at: {in_path}") from e

    def fetch_file(self, in_path, out_path):
        """fetch a file from jail to local"""
        super().fetch_file(in_path, out_path)
        display.vvv(f"FETCH {in_path} TO {out_path}", host=self.jail)

        in_path = shlex_quote(self._prefix_login_path(in_path))
        try:
            p = self._buffered_exec_command(f"dd if={in_path} bs={BUFSIZE}")
        except OSError as e:
            raise AnsibleError("jail connection requires dd command in the jail") from e

        with open(to_bytes(out_path, errors="surrogate_or_strict"), "wb+") as out_file:
            try:
                chunk = p.stdout.read(BUFSIZE)
                while chunk:
                    out_file.write(chunk)
                    chunk = p.stdout.read(BUFSIZE)
            except Exception as e:
                traceback.print_exc()
                raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
            stdout, stderr = p.communicate()
            if p.returncode != 0:
                raise AnsibleError(
                    f"failed to transfer file {in_path} to {out_path}:\n{to_native(stdout)}\n{to_native(stderr)}"
                )

    def close(self):
        """terminate the connection; nothing to do here"""
        super().close()
        self._connected = False



# Copyright (c) 2026, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: acme_dns_persist_record_parse
short_description: Parse a DNS record for ACME C(dns-persist-01) challenges
author: Felix Fontein (@felixfontein)
version_added: 3.2.0
description:
  - Parse the content for a ACME C(dns-persist-01) DNS TXT record V(_validation-persist.<domain>).
  - This filter conforms to the L(acme-dns-persist draft 01, https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html).
    Note that the supported draft version can change at any time,
    and changes will only be considered breaking once the draft reached RFC status.
options:
  _input:
    description:
      - The DNS TXT record entry.
    type: string
    required: true
seealso:
  - module: community.crypto.acme_certificate
  - module: community.crypto.acme_certificate_order_create
  - module: community.crypto.acme_certificate_order_validate
"""

EXAMPLES = r"""
---
- name: Create _validation-persist.<domain> TXT record contents
  ansible.builtin.debug:
    msg: "{{ record | community.crypto.acme_dns_persist_record_parse }}"
  var:
    record: >-
      letsencrypt.org;
      accounturi=https://acme-v02.api.letsencrypt.org/acme/acct/1234;
      policy=wildcard;
      persistUntil=1774813004

- name: Create _validation-persist.<domain> TXT record for example.com
  community.dns.hetzner_dns_record_set:
    prefix: _validation-persist
    zone_name: example.com
    value:
      - >-
        {{
          'letsencrypt.org' | community.crypto.acme_dns_persist_record(
            account_uri='https://acme-v02.api.letsencrypt.org/acme/acct/4321',
            persist_until='20190331202428Z',
          )
        }}
"""

RETURN = r"""
_value:
  description:
    - The content for the V(_validation-persist.<domain>) TXT record.
  type: dictionary
  contains:
    issuer_domain_name:
      type: string
      description:
        - The issuer domain name.
      sample: letsencrypt.org
    account_uri:
      type: string
      description:
        - The ACME account URI.
    policy:
      description:
        - The validation scope.
        - Is V(null) if not present.
      type: string
    persist_until:
      description:
        - Until when the record is valid.
        - This is a UNIX timestamp, that is the number of seconds since January 1st, 1970, in UTC.
        - Is V(null) if V(persistUntil) is not present.
      type: string
    persist_until_str:
      description:
        - A ASN.1 string representation of RV(_value.persist_until).
        - Is V(null) if V(persistUntil) is not present.
      type: string
"""

import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError

from ansible_collections.community.crypto.plugins.module_utils._caa import (
    parse_issue_value,
)
from ansible_collections.community.crypto.plugins.module_utils._time import (
    from_epoch_seconds,
)

TIMESTAMP_FORMAT = "%Y%m%d%H%M%SZ"


def acme_dns_persist_record_parse(
    record_value: t.Any,
) -> dict[str, t.Any]:
    if not isinstance(record_value, str):
        raise AnsibleFilterError(
            "The input for the community.crypto.acme_dns_persist_record_parse filter"
            f" must be a string; got {type(record_value)} instead"
        )
    try:
        domain_name, pairs = parse_issue_value(record_value)
    except ValueError as exc:
        raise AnsibleFilterError(
            "community.crypto.acme_dns_persist_record_parse filter could not parse"
            f" value: {exc}"
        ) from exc
    values = dict(pairs)
    if domain_name is None:
        raise AnsibleFilterError(
            "community.crypto.acme_dns_persist_record_parse filter: domain name not present"
        )
    try:
        account_uri = values.pop("accounturi")
    except KeyError:
        raise AnsibleFilterError(
            "community.crypto.acme_dns_persist_record_parse filter: cannot find account URI"
        ) from None
    policy = values.pop("policy", None)
    if policy is not None:
        policy = policy.lower()
        # TODO unknown policy
    persist_until_v = values.pop("persistUntil", None)
    persist_until: int | None = None
    persist_until_str: str | None = None
    if persist_until_v is not None:
        try:
            persist_until = int(persist_until_v)
        except ValueError as exc:
            raise AnsibleFilterError(
                f"community.crypto.acme_dns_persist_record_parse filter: error when parsing persistUntil: {exc}"
            ) from None
        persist_until_str = from_epoch_seconds(
            persist_until, with_timezone=True
        ).strftime(TIMESTAMP_FORMAT)
    result: dict[str, t.Any] = {
        "issuer_domain_name": domain_name,
        "account_uri": account_uri,
        "policy": policy,
        "persist_until": persist_until,
        "persist_until_str": persist_until_str,
    }
    # TODO values not empty
    return result


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "acme_dns_persist_record_parse": acme_dns_persist_record_parse,
        }



#!/usr/bin/python
# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
#
from __future__ import annotations

DOCUMENTATION = r"""
author:
  - Naval Patel (@navalkp)
  - Prashant Bhosale (@prabhosa)
module: lxca_nodes
short_description: Custom module for lxca nodes inventory utility
description:
  - This module returns/displays a inventory details of nodes.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  uuid:
    description: UUID of device, this is string with length greater than 16.
    type: str

  command_options:
    description: Options to filter nodes information.
    default: nodes
    choices:
      - nodes
      - nodes_by_uuid
      - nodes_by_chassis_uuid
      - nodes_status_managed
      - nodes_status_unmanaged
    type: str

  chassis:
    description: UUID of chassis, this is string with length greater than 16.
    type: str

extends_documentation_fragment:
  - community.general._lxca_common
  - community.general._attributes
"""

EXAMPLES = r"""
# get all nodes info
- name: Get nodes data from LXCA
  community.general.lxca_nodes:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    command_options: nodes

# get specific nodes info by uuid
- name: Get nodes data from LXCA
  community.general.lxca_nodes:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    uuid: "3C737AA5E31640CE949B10C129A8B01F"
    command_options: nodes_by_uuid

# get specific nodes info by chassis uuid
- name: Get nodes data from LXCA
  community.general.lxca_nodes:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    chassis: "3C737AA5E31640CE949B10C129A8B01F"
    command_options: nodes_by_chassis_uuid

# get managed nodes
- name: Get nodes data from LXCA
  community.general.lxca_nodes:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    command_options: nodes_status_managed

# get unmanaged nodes
- name: Get nodes data from LXCA
  community.general.lxca_nodes:
    login_user: USERID
    login_password: Password
    auth_url: "https://10.243.15.168"
    command_options: nodes_status_unmanaged
"""

RETURN = r"""
result:
  description: Nodes detail from lxca.
  returned: always
  type: dict
  sample:
    nodeList:
      - machineType: '6241'
        model: 'AC1'
        type: 'Rack-TowerServer'
        uuid: '118D2C88C8FD11E4947B6EAE8B4BDCDF'
          # bunch of properties
      - machineType: '8871'
        model: 'AC1'
        type: 'Rack-TowerServer'
        uuid: '223D2C88C8FD11E4947B6EAE8B4BDCDF'
          # bunch of properties
        # Multiple nodes details
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lxca_common import (
    LXCA_COMMON_ARGS,
    connection_object,
    has_pylxca,
)

try:
    from pylxca import nodes
except ImportError:
    pass


UUID_REQUIRED = "UUID of device is required for nodes_by_uuid command."
CHASSIS_UUID_REQUIRED = "UUID of chassis is required for nodes_by_chassis_uuid command."


def _nodes(module, lxca_con):
    return nodes(lxca_con)


def _nodes_by_uuid(module, lxca_con):
    if not module.params["uuid"]:
        module.fail_json(msg=UUID_REQUIRED)
    return nodes(lxca_con, module.params["uuid"])


def _nodes_by_chassis_uuid(module, lxca_con):
    if not module.params["chassis"]:
        module.fail_json(msg=CHASSIS_UUID_REQUIRED)
    return nodes(lxca_con, chassis=module.params["chassis"])


def _nodes_status_managed(module, lxca_con):
    return nodes(lxca_con, status="managed")


def _nodes_status_unmanaged(module, lxca_con):
    return nodes(lxca_con, status="unmanaged")


def setup_module_object():
    """
    this function merge argument spec and create ansible module object
    :return:
    """
    args_spec = dict(LXCA_COMMON_ARGS)
    args_spec.update(INPUT_ARG_SPEC)
    module = AnsibleModule(argument_spec=args_spec, supports_check_mode=False)

    return module


FUNC_DICT = {
    "nodes": _nodes,
    "nodes_by_uuid": _nodes_by_uuid,
    "nodes_by_chassis_uuid": _nodes_by_chassis_uuid,
    "nodes_status_managed": _nodes_status_managed,
    "nodes_status_unmanaged": _nodes_status_unmanaged,
}


INPUT_ARG_SPEC = dict(
    command_options=dict(
        default="nodes",
        choices=["nodes", "nodes_by_uuid", "nodes_by_chassis_uuid", "nodes_status_managed", "nodes_status_unmanaged"],
    ),
    uuid=dict(),
    chassis=dict(),
)


def execute_module(module):
    """
    This function invoke commands
    :param module: Ansible module object
    """
    try:
        with connection_object(module) as lxca_con:
            result = FUNC_DICT[module.params["command_options"]](module, lxca_con)
            module.exit_json(changed=False, msg=f"Success {module.params['command_options']} result", result=result)
    except Exception as exception:
        error_msg = "; ".join(exception.args)
        module.fail_json(msg=error_msg, exception=traceback.format_exc())


def main():
    module = setup_module_object()
    has_pylxca(module)
    execute_module(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_workspace
version_added: "3.15.0"
short_description: Create, Update or Delete an Azure Machine Learning Workspace
description:
    - Create, Update or Delete an Azure Machine Learning Workspace.
options:
    name:
        description:
            - Name of the Azure ML workspace.
        type: str
        required: true
    display_name:
        description:
            - Display name for the workspace.
        type: str
        required: false
    description:
        description:
            - Description of the Azure ML workspace.
        type: str
        required: false
    resource_definition:
        description:
            - Workspace definition in YAML
        type: str
        required: false
    kind:
        description:
            - Specifies the workspace as a specific kind.
        type: str
        choices:
            - hub
            - project
        required: false
    update_dependent_resources:
        description:
            - Specifying update_dependent_resources, gives your
              consent to update the workspace dependent
              resources. Updating the workspace-attached Azure
              Container Registry or Application Insights resource
              may break lineage of previous jobs, deployed
              inference endpoints, or your ability to rerun
              earlier jobs in this workspace.
        type: bool
        default: false
    public_network_access:
        description:
            - Allow public endpoint connectivity when a workspace
              is private link enabled.
        type: str
        choices:
            - Disabled
            - Enabled
        default: Disabled
    image_build_compute:
        description:
            - The name of the compute target to use for building
              environment Docker images when the container
              registry is behind a VNet.
        type: str
        required: false
    storage_account:
        description:
            - ARM id of the storage account associated with this workspace.
        type: str
        required: false
    key_vault:
        description:
            - ARM id of the key vault associated with this workspace.
        type: str
        required: false
    application_insights:
        description:
            - ARM id of the application insights associated with
              this workspace.
        type: str
        required: false
    container_registry:
        description:
            - ARM id of the container registry associated with
              this workspace.
        type: str
        required: false
    hub_id:
        description:
            - Only applies to O(kind=Project). An ARM ID which defines the
              parent hub of this project.
        type: str
        required: false
    enable_data_isolation:
        description:
            - A flag to determine if a workspace has data
              isolation enabled. The flag can only be set at the
              creation stage, it can't be updated.
        type: bool
        default: false
    primary_user_assigned_identity:
        description:
            - ARM identifier of primary user assigned managed
              identity, in case multiple ones are specified. Also
              the default managed identity for clusterless compute.
        type: str
        required: false
    managed_network:
        description:
            - Managed Network Isolation Mode for the workspace.
        type: str
        choices:
            - disabled
            - allow_internet_outbound
            - allow_only_approved_outbound
        required: false
    network_acls:
        description:
            - List of inbound IP rules
        required: false
        type: list
        elements: str
    provision_network_now:
        description:
            - Set to trigger the provisioning of the
              managed network when creating a worksapce with the
              manged network enabled, or else it does nothing.
        type: bool
        default: false
    system_datastores_auth_mode:
        description:
            - Specifies the auth mode for the system data stores.
        type: str
        choices:
            - accesskey
            - identity
    allow_roleassignment_on_rg:
        description:
            - A flag to determine if a workspace could
              have role assignments on resource group level.
        type: bool
        default: true
    location:
        description:
            - The location to be used for the new workspace.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    default_resource_group:
        description:
            - Only applies to O(kind=hub). If set, then child projects of this
              hub will have their resource group set this by default.
        required: false
        type: str
    all_resources:
        description:
            - Only applies to O(state=absent). Delete all the dependent resources
              associated with the workspace (Azure Storage account,
              Azure Container Registry, Azure Application Insights, Azure Key Vault).
        type: bool
        default: false
    permanently_delete:
        description:
            - Only applies to O(state=absent). Workspaces are soft-deleted
              state by default to allow recovery of workspace data. Set this flag
              to override the soft-delete behavior and permanently delete your
              workspace.
        type: bool
        default: false
    state:
        description:
            - State of the Workspace. Use C(present) to create
              or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Workspace
  azure.azcollection.azure_rm_ml_workspace:
    location: eastus
    name: MyWorkspace
    description: My Workspace created by Ansible
    resource_group: myResourceGroup
    resource_definition: |
      managed_network:
        isolation_mode: allow_only_approved_outbound
        outbound_rules:
        - name: microsoft
          destination: 'microsoft.com'
          type: fqdn
        - name: servicetag-w-prefixes
          destination:
            port_ranges: 80, 8080-8089
            protocol: TCP
            service_tag: sometag
            address_prefixes: ["168.63.129.16","10.0.0.0/24"]
          type: service_tag
    tags:
      createdByToolkit: Ansible
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_workspace:
    description:
        - Workspace that was just created or updated.
    returned: always
    type: dict
    sample: {
        "allow_roleassignment_on_rg": false,
        "application_insights": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.insights/components/workspacinsightsxxxxxxxx",
        "description": "Workspace Integration Tests",
        "discovery_url": "https://westus3.api.azureml.ms/discovery",
        "display_name": "workspace-xxxxxxx-xxx",
        "enable_data_isolation": false,
        "hbi_workspace": false,
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx",
        "identity": {
            "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "type": "system_assigned"
        },
        "key_vault": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Keyvault/vaults/workspackeyvaultxxxxxxxx",
        "location": "westus3",
        "managed_network": {
            "firewall_sku": "standard",
            "isolation_mode": "allow_only_approved_outbound",
            "network_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "outbound_rules": [
                {
                    "category": "user_defined",
                    "destination": "microsoft.com",
                    "name": "microsoft",
                    "status": "Inactive",
                    "type": "fqdn"
                },
                {
                    "category": "user_defined",
                    "destination": {
                        "address_prefixes": [
                            "168.63.129.16",
                            "10.0.0.0/24"
                        ],
                        "port_ranges": "80, 8080-8089",
                        "protocol": "TCP",
                        "service_tag": "sometag"
                    },
                    "name": "servicetag-w-prefixes",
                    "status": "Inactive",
                    "type": "service_tag"
                 }
            ],
            "status": {
                "spark_ready": false,
                "status": "Inactive"
            }
        },
        "mlflow_tracking_uri": "azureml://westus3.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx",
        "name": "workspace-xxxxxxx-xxx",
        "network_acls": {
            "default_action": "Allow",
            "ip_rules": []
        },
        "provision_network_now": false,
        "public_network_access": "Disabled",
        "resource_group": "test-ml-workspace",
        "serverless_compute": {"no_public_ip": false},
        "storage_account": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Storage/storageAccounts/workspacstoragexxxxxxxxx",
        "system_datastores_auth_mode": "accesskey",
        "tags": {
            "createdByToolkit": "sdk-v2-1.31.0",
            "foo": "bar"
        }
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_workspace
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLWorkspace(MLClientCommon):

    PARAM_OVERRIDES = ['name',
                       'display_name',
                       'description',
                       'tags',
                       'location',
                       'image_build_compute',
                       'public_network_access',
                       'storage_account',
                       'key_vault',
                       'application_insights',
                       'container_registry',
                       'primary_user_assigned_identity',
                       'provision_network_now',
                       'enable_data_isolation',
                       'hub_id',
                       'default_resource_group',
                       'kind',
                       'system_datastores_auth_mode',
                       'allow_roleassignment_on_rg',
                       ]

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True
            ),
            display_name=dict(
                type='str',
                required=False,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            kind=dict(
                type='str',
                choices=['hub',
                         'project'],
                required=False,
            ),
            update_dependent_resources=dict(
                type='bool',
                default=False,
            ),
            public_network_access=dict(
                type='str',
                choices=['Disabled', 'Enabled'],
                default='Disabled',
            ),
            image_build_compute=dict(
                type='str',
                required=False,
            ),
            storage_account=dict(
                type='str',
                required=False,
            ),
            key_vault=dict(
                type='str',
                required=False,
                no_log=False,
            ),
            application_insights=dict(
                type='str',
                required=False,
            ),
            container_registry=dict(
                type='str',
                required=False,
            ),
            hub_id=dict(
                type='str',
                required=False,
            ),
            enable_data_isolation=dict(
                type='bool',
                default=False,
            ),
            primary_user_assigned_identity=dict(
                type='str',
                required=False,
            ),
            managed_network=dict(
                type='str',
                choices=['disabled',
                         'allow_internet_outbound',
                         'allow_only_approved_outbound'],
                required=False,
            ),
            network_acls=dict(
                type='list',
                required=False,
                elements='str',
            ),
            provision_network_now=dict(
                type='bool',
                default=False,
            ),
            system_datastores_auth_mode=dict(
                type='str',
                choices=['accesskey',
                         'identity'],
                required=False,
            ),
            allow_roleassignment_on_rg=dict(
                type='bool',
                default=True,
            ),
            location=dict(
                type='str',
                required=False
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            default_resource_group=dict(
                type='str',
                required=False
            ),
            all_resources=dict(
                type='bool',
                default=False
            ),
            permanently_delete=dict(
                type='bool',
                default=False
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
        )

        self._client = None
        self.ml_workspace = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_workspace={}
        )

        super(AzureRMMLWorkspace, self).__init__(self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 )

    def exec_module(self, **kwargs):

        params_override = []
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])
            if kwargs[key] is not None and key in self.PARAM_OVERRIDES:
                params_override.append({key: kwargs[key]})

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        # Process managed_network specially
        if self.managed_network:
            params_override.append({"managed_network": {"isolation_mode": self.managed_network}})

        # Process network_acls specially
        if self.network_acls:
            network_acls = {
                "default_action": "Deny",
                "ip_rules": [{"value": ip} for ip in self.network_acls]
            }
        else:
            network_acls = {
                "default_action": "Allow",
                "ip_rules": []
            }
        params_override.append({'network_acls': network_acls})

        changed = False
        # Generate ml_workspace based on ansible args passed in.
        ml_workspace = load_workspace(resource_definition,
                                      params_override=params_override)
        try:
            # Attempt to retrieve ml_workspace based on name
            ml_workspace_info = self.client.workspaces.get(self.name)
        except ResourceNotFoundError:
            ml_workspace_info = None

        if self.state == 'present':
            if ml_workspace_info:
                # Update
                ml_workspace_info = self.entity_to_dict(ml_workspace_info)
                ml_workspace_info = self.filter_required(ml_workspace_info)
                changed = not self.default_compare({},
                                                   self.filter_required(
                                                       self.entity_to_dict(ml_workspace)),
                                                   ml_workspace_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.workspaces.begin_update(ml_workspace,
                                                                   update_dependent_resources=self.update_dependent_resources
                                                                   )
                    ml_workspace_object = self.get_poller_result(response)
                    ml_workspace_info = self.entity_to_dict(ml_workspace_object)

            else:
                # Create
                changed = True
                if not self.check_mode:
                    response = self.client.workspaces.begin_create(
                        workspace=ml_workspace,
                        update_dependent_resources=self.update_dependent_resources)
                    ml_workspace_object = self.get_poller_result(response)
                    ml_workspace_info = self.entity_to_dict(ml_workspace_object)
        else:
            # Delete
            if ml_workspace_info:
                changed = True
                if not self.check_mode:
                    response = self.client.workspaces.begin_delete(
                        name=self.name,
                        delete_dependent_resources=self.all_resources,
                        permanently_delete=self.permanently_delete)
                    ml_workspace_info = self.get_poller_result(response)

        self.results['ml_workspace'] = ml_workspace_info
        self.results['changed'] = changed
        return self.results


def main():
    AzureRMMLWorkspace()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# SPDX-License-Identifier: GPL-3.0-or-later

# Copyright © Thorsten Glaser <tglaser@b1-systems.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = r"""
---
module: kea_command
short_description: Submits generic command to ISC KEA server on target
description:
  - Submits a command to the JSON API of an ISC KEA server running on the target and obtains the result.
  - This module supports sending arbitrary commands and returns the server response unchecked;
    while it would be possible to write individual modules for specific KEA service commands,
    that approach would not scale, as the FOSS hooks alone provide dozens of commands.
  - Between sending the command and parsing the result status, RV(ignore:changed) registers as V(true) if an error occurs,
    to err on the safe side.
version_added: '12.0.0'
author: Thorsten Glaser (@mirabilos)
options:
  command:
    description:
      - The name of the command to send, for example V(status-get).
    required: true
    type: str
  arguments:
    description:
      - The arguments sent along with the command, if any.
      - Use V({}) to send an empty arguments dict/object instead of omitting it.
    type: dict
  rv_unchanged:
    description:
      - A list of C(result) codes to indicate success but unchanged system state.
      - Set this to V([0]) for most acquisition commands.
      - Use V([3]) for O(command=lease4-del) and similar which have a separate code for this.
      - Any C(result) codes not listed in either O(rv_unchanged) or O(rv_changed) are interpreted as indicating an error result.
      - O(rv_unchanged) has precedence over O(rv_changed) if a result code is in both lists.
    type: list
    elements: int
    default: []
  rv_changed:
    description:
      - A list of C(result) codes to indicate success and changed system state.
      - Omit this for most acquisition commands.
      - Set it to V([0]) for O(command=lease4-del) and similar which return changed system state that way.
      - Any C(result) codes not listed in either O(rv_unchanged) or O(rv_changed) are interpreted as indicating an error result.
      - O(rv_unchanged) has precedence over O(rv_changed) if a result code is in both lists.
    type: list
    elements: int
    default: []
  socket:
    description:
      - The full pathname of the Unix Domain Socket to connect to.
      - The default value is suitable for C(kea-dhcp4-server) on Debian trixie.
      - This module directly interfaces using UDS; the HTTP wrappers are not supported.
    type: path
    default: /run/kea/kea4-ctrl-socket
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.platform
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  platform:
    support: full
    platforms: posix
"""

EXAMPLES = r"""
vars:
  ipaddr: "192.168.123.45"
  hwaddr: "00:00:5E:00:53:00"
tasks:

  # an example for a request acquiring information
  - name: Get KEA DHCP6 status
    kea_command:
      command: status-get
      rv_unchanged: [0]
      socket: /run/kea/kea6-ctrl-socket
    register: kea6_status
  - name: Display registered status result
    ansible.builtin.debug:
      msg: KEA DHCP6 running on PID {{ kea6_status.response.arguments.pid }}

  # an example for requests modifying state
  - name: Remove existing leases for {{ ipaddr }}, if any
    kea_command:
      command: lease4-del
      arguments:
        ip-address: "{{ ipaddr }}"
      rv_changed: [0]
      rv_unchanged: [3]
  - name: Add DHCP lease for {{ ipaddr }}
    kea_command:
      command: lease4-add
      arguments:
        ip-address: "{{ ipaddr }}"
        hw-address: "{{ hwaddr }}"
      rv_changed: [0]
"""

RETURN = r"""
response:
  description: The server JSON response.
  returned: when available
  type: dict
"""

import json
import os
import socket
import traceback

from ansible.module_utils.basic import AnsibleModule

# default buffer size for socket I/O
BUFSIZ = 8192


def _parse_constant(s):
    raise ValueError(f'Invalid JSON: "{s}"')


def main():
    module = AnsibleModule(
        argument_spec=dict(
            command=dict(type="str", required=True),
            arguments=dict(type="dict"),
            rv_unchanged=dict(type="list", elements="int", default=[]),
            rv_changed=dict(type="list", elements="int", default=[]),
            socket=dict(type="path", default="/run/kea/kea4-ctrl-socket"),
        ),
    )

    cmd = {}
    cmd["command"] = module.params["command"]
    if module.params["arguments"] is not None:
        cmd["arguments"] = module.params["arguments"]
    cmdstr = json.dumps(cmd, ensure_ascii=True, allow_nan=False, indent=None, separators=(",", ":"), sort_keys=True)
    rvok = module.params["rv_unchanged"]
    rvch = module.params["rv_changed"]
    sockfn = module.params["socket"]

    r = {"changed": False}
    rsp = b""

    if not os.path.exists(sockfn):
        r["msg"] = f"socket ({sockfn}) does not exist"
        module.fail_json(**r)

    phase = "opening"
    try:
        with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as sock:
            phase = "connecting"
            sock.connect(sockfn)
            # better safe in case anything fails…
            r["changed"] = True
            phase = "writing"
            sock.sendall(cmdstr.encode("ASCII"))
            phase = "reading"
            while True:
                rspnew = sock.recv(BUFSIZ)
                if len(rspnew) == 0:
                    break
                rsp += rspnew
            phase = "closing"
    except OSError as ex:
        r["msg"] = f"error {phase} socket ({sockfn}): {ex}"
        r["exception"] = traceback.format_exc()
        module.fail_json(**r)

    # 15 is the length of the minimum response {"response":0} as formatted by KEA
    if len(rsp) < 15:
        r["msg"] = f"unrealistically short response {rsp!r}"
        module.fail_json(**r)

    try:
        r["response"] = json.loads(rsp, parse_constant=_parse_constant)
    except ValueError as ex:
        r["msg"] = f"error parsing JSON response: {ex}"
        r["exception"] = traceback.format_exc()
        module.fail_json(**r)
    if not isinstance(r["response"], dict):
        r["msg"] = "bogus JSON response (JSONObject expected)"
        module.fail_json(**r)
    if "result" not in r["response"]:
        r["msg"] = "bogus JSON response (missing result)"
        module.fail_json(**r)
    res = r["response"]["result"]
    if not isinstance(res, int):
        r["msg"] = "bogus JSON response (non-integer result)"
        module.fail_json(**r)

    if res in rvok:
        r["changed"] = False
    elif res not in rvch:
        r["msg"] = f"failure result (code {res})"
        module.fail_json(**r)

    module.exit_json(**r)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Gaspard Micol (@gmicol) <gmicol@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_netflow_exporter_policy
short_description: Manage Netflow Exporter Policy (netflow:ExporterPol)
description:
- Manage Netflow Exporter Policies for tenants on Cisco ACI fabrics.
- Exporter information for bootstrapping the netflow Collection agent.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  netflow_exporter_policy:
    description:
    - The name of the Netflow Exporter Policy.
    type: str
    aliases: [ netflow_exporter, netflow_exporter_name, name ]
  dscp:
    description:
    - The IP DSCP value.
    - The APIC defaults to C(CS2) when unset during creation.
      It defaults to C(VA) for APIC versions 4.2 or prior.
    type: str
    choices: [ AF11, AF12, AF13, AF21, AF22, AF23, AF31, AF32, AF33, AF41, AF42, AF43, CS0, CS1, CS2, CS3, CS4, CS5, CS6, CS7, EF, VA, unspecified ]
  destination_address:
    description:
    - The remote node destination IP address.
    type: str
  destination_port:
    description:
    - The remote node destination port.
    - Accepted values are any valid TCP/UDP port range.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
  source_ip_type:
    description:
    - The type of Exporter source IP Address.
    - It can be one of the available management IP Address for a given leaf or a custom IP Address.
    type: str
    choices: [ custom_source_ip, inband_management_ip, out_of_band_management_ip, ptep ]
  custom_source_address:
    description:
    - The custom source IP address.
    - It can only be used if O(source_ip_type=custom_source_ip).
    type: str
  associated_epg:
   description:
   - The associated EPG.
   - To remove the current associated EPG, pass an empty dictionary.
   type: dict
   aliases: [ epg ]
   suboptions:
      tenant:
        description:
        - The name of the tenant to which the associated AP/EPG  belong.
        type: str
      ap:
        description:
        - The name of the associated Application Profile to which the associated EPG belongs.
        type: str
      epg:
        description:
        - The name of the associated EPG.
        type: str
  associated_extepg:
    description:
    - The associated external EPG.
    - To remove the current associated external EPG, pass an empty dictionary.
    type: dict
    aliases: [ external_epg, associated_external_epg ]
    suboptions:
      tenant:
        description:
        - The name of the tenant to which the associated L3Out/external EPG belong.
        type: str
      l3out:
        description:
        - The name of the L3Out to which the associated external EPG belongs.
        type: str
      extepg:
        description:
        - The name of the associated EPG.
        type: str
  associated_vrf:
    description:
    - The associated VRF.
    - To remove the current associated VRF, pass an empty dictionary.
    type: dict
    aliases: [ vrf, context, associated_context ]
    suboptions:
      tenant:
        description:
        - The name of the tenant to which the associated VRF belongs.
        type: str
      vrf:
        description:
        - The name of the associated VRF.
        type: str
  description:
    description:
    - The description for the Netflow Exporter Policy.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) can be used for this.
- The I(associated_epg) and I(associated_extepg) are mutually exclusive.
- If the I(associated_epg) is used, the I(epg), the(tenant) and
  the I(ap) must exist before using this module in your play book.
  The M(cisco.aci.aci_epg) and the M(cisco.aci.aci_ap) can be used for this.
- If the I(associated_extepg) is used, the I(extepg), the(tenant) and
  the I(l3out) must exist before using this module in your play book.
  The M(cisco.aci.aci_l3out_extepg) and the M(cisco.aci.aci_l3out) can be used for this.
- If the I(associated_vrf) is used, the I(vrf) and the I(tenant) must exist
  before using this module in your play book.
  The M(cisco.aci.aci_vrf) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_vrf
- module: cisco.aci.aci_ap
- module: cisco.aci.aci_epg
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_extepg
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(netflow:ExporterPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Gaspard Micol (@gmicol)
"""

EXAMPLES = r"""
- name: Add a new Netflow Exporter Policy
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    netflow_exporter_policy: my_netflow_exporter_policy
    dscp: CS2
    destination_address: 11.11.11.1
    destination_port: 25
    source_ip_type: custom_source_ip
    custom_source_address: 11.11.11.2
    associated_epg:
      tenant: my_tenant
      ap: my_ap
      epg: my_epg
    associated_vrf:
      tenant: my_tenant
      vrf: my_vrf
    state: present
  delegate_to: localhost

- name: Remove associated EPG and VRF from the new Netflow Exporter Policy
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    netflow_exporter_policy: my_netflow_exporter_policy
    associated_epg: {}
    associated_vrf: {}
  delegate_to: localhost

- name: Query a Netflow Exporter Policy
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    netflow_exporter_policy: my_netflow_exporter_policy
    state: query
  delegate_to: localhost

- name: Query all Netflow Exporter Policies in my_tenant
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    state: query
  delegate_to: localhost

- name: Query all Netflow Exporter Policies
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Delete a Netflow Exporter Policy
  cisco.aci.aci_netflow_exporter_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    netflow_exporter_policy: my_netflow_exporter_policy
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    aci_annotation_spec,
    aci_owner_spec,
    aci_contract_dscp_spec,
    associated_netflow_exporter_epg_spec,
    associated_netflow_exporter_extepg_spec,
    associated_netflow_exporter_vrf_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import MATCH_SOURCE_IP_TYPE_NETFLOW_EXPORTER_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        netflow_exporter_policy=dict(type="str", aliases=["netflow_exporter", "netflow_exporter_name", "name"]),
        dscp=dict((k, aci_contract_dscp_spec()[k]) for k in aci_contract_dscp_spec() if k != "aliases"),
        destination_address=dict(type="str"),
        destination_port=dict(type="str"),
        source_ip_type=dict(type="str", choices=list(MATCH_SOURCE_IP_TYPE_NETFLOW_EXPORTER_MAPPING.keys())),
        custom_source_address=dict(type="str"),
        associated_epg=dict(type="dict", aliases=["epg"], options=associated_netflow_exporter_epg_spec()),
        associated_extepg=dict(type="dict", aliases=["external_epg", "associated_external_epg"], options=associated_netflow_exporter_extepg_spec()),
        associated_vrf=dict(type="dict", aliases=["vrf", "associated_context", "context"], options=associated_netflow_exporter_vrf_spec()),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "netflow_exporter_policy"]],
            ["state", "present", ["tenant", "netflow_exporter_policy", "destination_address", "destination_port"]],
        ],
        mutually_exclusive=[["associated_epg", "associated_extepg"]],
    )

    tenant = module.params.get("tenant")
    description = module.params.get("description")
    netflow_exporter_policy = module.params.get("netflow_exporter_policy")
    dscp = module.params.get("dscp")
    destination_address = module.params.get("destination_address")
    destination_port = module.params.get("destination_port")
    source_ip_type = MATCH_SOURCE_IP_TYPE_NETFLOW_EXPORTER_MAPPING.get(module.params.get("source_ip_type"))
    custom_source_address = module.params.get("custom_source_address")
    associated_epg = module.params.get("associated_epg")
    associated_extepg = module.params.get("associated_extepg")
    associated_vrf = module.params.get("associated_vrf")
    state = module.params.get("state")

    aci = ACIModule(module)

    child_classes = ["netflowRsExporterToCtx", "netflowRsExporterToEPg"]

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="netflowExporterPol",
            aci_rn="exporterpol-{0}".format(netflow_exporter_policy),
            module_object=netflow_exporter_policy,
            target_filter={"name": netflow_exporter_policy},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if associated_vrf is not None:
            if all(value is None for value in associated_vrf.values()) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("netflowExporterPol", {}).get("children", {}):
                    if child.get("netflowRsExporterToCtx"):
                        child_configs.extend([dict(netflowRsExporterToCtx=dict(attributes=dict(status="deleted")))])
            elif all(value is not None for value in associated_vrf.values()):
                child_configs.extend(
                    [
                        dict(
                            netflowRsExporterToCtx=dict(
                                attributes=dict(tDn="uni/tn-{0}/ctx-{1}".format(associated_vrf.get("tenant"), associated_vrf.get("vrf")))
                            )
                        ),
                    ]
                )
        if associated_epg is not None:
            if all(value is None for value in associated_epg.values()) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("netflowExporterPol", {}).get("children", {}):
                    if child.get("netflowRsExporterToEPg"):
                        child_configs.extend([dict(netflowRsExporterToEPg=dict(attributes=dict(status="deleted")))])
            elif all(value is not None for value in associated_epg.values()):
                child_configs.extend(
                    [
                        dict(
                            netflowRsExporterToEPg=dict(
                                attributes=dict(
                                    tDn="uni/tn-{0}/ap-{1}/epg-{2}".format(associated_epg.get("tenant"), associated_epg.get("ap"), associated_epg.get("epg"))
                                )
                            )
                        ),
                    ]
                )
        elif associated_extepg is not None:
            if all(value is None for value in associated_extepg.values()) and isinstance(aci.existing, list) and len(aci.existing) > 0:
                for child in aci.existing[0].get("netflowExporterPol", {}).get("children", {}):
                    if child.get("netflowRsExporterToEPg"):
                        child_configs.extend([dict(netflowRsExporterToEPg=dict(attributes=dict(status="deleted")))])
            elif all(value is not None for value in associated_extepg.values()):
                child_configs.extend(
                    [
                        dict(
                            netflowRsExporterToEPg=dict(
                                attributes=dict(
                                    tDn="uni/tn-{0}/out-{1}/instP-{2}".format(
                                        associated_extepg.get("tenant"), associated_extepg.get("l3out"), associated_extepg.get("extepg")
                                    )
                                )
                            )
                        ),
                    ]
                )
        aci.payload(
            aci_class="netflowExporterPol",
            class_config=dict(
                name=netflow_exporter_policy,
                descr=description,
                dscp=dscp,
                dstAddr=destination_address,
                dstPort=destination_port,
                sourceIpType=source_ip_type,
                srcAddr=custom_source_address,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="netflowExporterPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright 2014 Benjamin Curtis <benjamin.curtis@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: honeybadger_deployment
author: "Benjamin Curtis (@stympy)"
short_description: Notify Honeybadger.io about app deployments
description:
  - Notify Honeybadger.io about app deployments (see U(http://docs.honeybadger.io/article/188-deployment-tracking)).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  token:
    type: str
    description:
      - API token.
    required: true
  environment:
    type: str
    description:
      - The environment name, typically V(production), V(staging), and so on.
    required: true
  user:
    type: str
    description:
      - The username of the person doing the deployment.
  repo:
    type: str
    description:
      - URL of the project repository.
  revision:
    type: str
    description:
      - A hash, number, tag, or other identifier showing what revision was deployed.
  url:
    type: str
    description:
      - Optional URL to submit the notification to.
    default: "https://api.honeybadger.io/v1/deploys"
  validate_certs:
    description:
      - If V(false), SSL certificates for the target URL are not validated. This should only be used on personally controlled
        sites using self-signed certificates.
    type: bool
    default: true
"""

EXAMPLES = r"""
- name: Notify Honeybadger.io about an app deployment
  community.general.honeybadger_deployment:
    token: AAAAAA
    environment: staging
    user: ansible
    revision: b6826b8
    repo: 'git@github.com:user/repo.git'
"""

RETURN = """#"""

import traceback
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

# ===========================================
# Module execution.
#


def main():
    module = AnsibleModule(
        argument_spec=dict(
            token=dict(required=True, no_log=True),
            environment=dict(required=True),
            user=dict(required=False),
            repo=dict(),
            revision=dict(),
            url=dict(default="https://api.honeybadger.io/v1/deploys"),
            validate_certs=dict(default=True, type="bool"),
        ),
        supports_check_mode=True,
    )

    params = {}

    if module.params["environment"]:
        params["deploy[environment]"] = module.params["environment"]

    if module.params["user"]:
        params["deploy[local_username]"] = module.params["user"]

    if module.params["repo"]:
        params["deploy[repository]"] = module.params["repo"]

    if module.params["revision"]:
        params["deploy[revision]"] = module.params["revision"]

    params["api_key"] = module.params["token"]

    url = module.params.get("url")

    # If we're in check mode, just exit pretending like we succeeded
    if module.check_mode:
        module.exit_json(changed=True)

    try:
        data = urlencode(params)
        response, info = fetch_url(module, url, data=data)
    except Exception as e:
        module.fail_json(msg=f"Unable to notify Honeybadger: {e}", exception=traceback.format_exc())
    else:
        if info["status"] == 201:
            module.exit_json(changed=True)
        else:
            module.fail_json(msg=f"HTTP result code: {info['status']} connecting to {url}")


if __name__ == "__main__":
    main()
